# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..epi import EpiReg


def test_EpiReg_inputs():
    input_map = dict(
        args=dict(
            argstr="%s",
        ),
        echospacing=dict(
            argstr="--echospacing=%f",
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        epi=dict(
            argstr="--epi=%s",
            extensions=None,
            mandatory=True,
            position=-4,
        ),
        fmap=dict(
            argstr="--fmap=%s",
            extensions=None,
        ),
        fmapmag=dict(
            argstr="--fmapmag=%s",
            extensions=None,
        ),
        fmapmagbrain=dict(
            argstr="--fmapmagbrain=%s",
            extensions=None,
        ),
        no_clean=dict(
            argstr="--noclean",
            usedefault=True,
        ),
        no_fmapreg=dict(
            argstr="--nofmapreg",
        ),
        out_base=dict(
            argstr="--out=%s",
            position=-1,
            usedefault=True,
        ),
        output_type=dict(),
        pedir=dict(
            argstr="--pedir=%s",
        ),
        t1_brain=dict(
            argstr="--t1brain=%s",
            extensions=None,
            mandatory=True,
            position=-2,
        ),
        t1_head=dict(
            argstr="--t1=%s",
            extensions=None,
            mandatory=True,
            position=-3,
        ),
        weight_image=dict(
            argstr="--weight=%s",
            extensions=None,
        ),
        wmseg=dict(
            argstr="--wmseg=%s",
            extensions=None,
        ),
    )
    inputs = EpiReg.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_EpiReg_outputs():
    output_map = dict(
        epi2str_inv=dict(
            extensions=None,
        ),
        epi2str_mat=dict(
            extensions=None,
        ),
        fmap2epi_mat=dict(
            extensions=None,
        ),
        fmap2str_mat=dict(
            extensions=None,
        ),
        fmap_epi=dict(
            extensions=None,
        ),
        fmap_str=dict(
            extensions=None,
        ),
        fmapmag_str=dict(
            extensions=None,
        ),
        fullwarp=dict(
            extensions=None,
        ),
        out_1vol=dict(
            extensions=None,
        ),
        out_file=dict(
            extensions=None,
        ),
        seg=dict(
            extensions=None,
        ),
        shiftmap=dict(
            extensions=None,
        ),
        wmedge=dict(
            extensions=None,
        ),
        wmseg=dict(
            extensions=None,
        ),
    )
    outputs = EpiReg.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..model import MELODIC


def test_MELODIC_inputs():
    input_map = dict(
        ICs=dict(
            argstr="--ICs=%s",
            extensions=None,
        ),
        approach=dict(
            argstr="-a %s",
        ),
        args=dict(
            argstr="%s",
        ),
        bg_image=dict(
            argstr="--bgimage=%s",
            extensions=None,
        ),
        bg_threshold=dict(
            argstr="--bgthreshold=%f",
        ),
        cov_weight=dict(
            argstr="--covarweight=%f",
        ),
        dim=dict(
            argstr="-d %d",
        ),
        dim_est=dict(
            argstr="--dimest=%s",
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        epsilon=dict(
            argstr="--eps=%f",
        ),
        epsilonS=dict(
            argstr="--epsS=%f",
        ),
        in_files=dict(
            argstr="-i %s",
            mandatory=True,
            position=0,
            sep=",",
        ),
        log_power=dict(
            argstr="--logPower",
        ),
        mask=dict(
            argstr="-m %s",
            extensions=None,
        ),
        max_restart=dict(
            argstr="--maxrestart=%d",
        ),
        maxit=dict(
            argstr="--maxit=%d",
        ),
        migp=dict(
            argstr="--migp",
        ),
        migpN=dict(
            argstr="--migpN %d",
        ),
        migp_factor=dict(
            argstr="--migp_factor %d",
        ),
        migp_shuffle=dict(
            argstr="--migp_shuffle",
        ),
        mix=dict(
            argstr="--mix=%s",
            extensions=None,
        ),
        mm_thresh=dict(
            argstr="--mmthresh=%f",
        ),
        no_bet=dict(
            argstr="--nobet",
        ),
        no_mask=dict(
            argstr="--nomask",
        ),
        no_mm=dict(
            argstr="--no_mm",
        ),
        non_linearity=dict(
            argstr="--nl=%s",
        ),
        num_ICs=dict(
            argstr="-n %d",
        ),
        out_all=dict(
            argstr="--Oall",
        ),
        out_dir=dict(
            argstr="-o %s",
            genfile=True,
        ),
        out_mean=dict(
            argstr="--Omean",
        ),
        out_orig=dict(
            argstr="--Oorig",
        ),
        out_pca=dict(
            argstr="--Opca",
        ),
        out_stats=dict(
            argstr="--Ostats",
        ),
        out_unmix=dict(
            argstr="--Ounmix",
        ),
        out_white=dict(
            argstr="--Owhite",
        ),
        output_type=dict(),
        pbsc=dict(
            argstr="--pbsc",
        ),
        rem_cmp=dict(
            argstr="-f %d",
        ),
        remove_deriv=dict(
            argstr="--remove_deriv",
        ),
        report=dict(
            argstr="--report",
        ),
        report_maps=dict(
            argstr="--report_maps=%s",
        ),
        s_con=dict(
            argstr="--Scon=%s",
            extensions=None,
        ),
        s_des=dict(
            argstr="--Sdes=%s",
            extensions=None,
        ),
        sep_vn=dict(
            argstr="--sep_vn",
        ),
        sep_whiten=dict(
            argstr="--sep_whiten",
        ),
        smode=dict(
            argstr="--smode=%s",
            extensions=None,
        ),
        t_con=dict(
            argstr="--Tcon=%s",
            extensions=None,
        ),
        t_des=dict(
            argstr="--Tdes=%s",
            extensions=None,
        ),
        tr_sec=dict(
            argstr="--tr=%f",
        ),
        update_mask=dict(
            argstr="--update_mask",
        ),
        var_norm=dict(
            argstr="--vn",
        ),
    )
    inputs = MELODIC.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_MELODIC_outputs():
    output_map = dict(
        out_dir=dict(),
        report_dir=dict(),
    )
    outputs = MELODIC.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
import pickle
import os.path as op

import numpy as np
import nibabel as nb
import networkx as nx

from ... import logging
from ...utils.filemanip import split_filename

from ..base import (
    BaseInterface,
    BaseInterfaceInputSpec,
    traits,
    File,
    TraitedSpec,
    Directory,
    OutputMultiPath,
    isdefined,
)

iflogger = logging.getLogger("nipype.interface")


def length(xyz, along=False):
    """
    Euclidean length of track line

    Parameters
    ----------
    xyz : array-like shape (N,3)
       array representing x,y,z of N points in a track
    along : bool, optional
       If True, return array giving cumulative length along track,
       otherwise (default) return scalar giving total length.

    Returns
    -------
    L : scalar or array shape (N-1,)
       scalar in case of `along` == False, giving total length, array if
       `along` == True, giving cumulative lengths.

    Examples
    --------
    >>> xyz = np.array([[1,1,1],[2,3,4],[0,0,0]])
    >>> expected_lens = np.sqrt([1+2**2+3**2, 2**2+3**2+4**2])
    >>> length(xyz) == expected_lens.sum()
    True
    >>> len_along = length(xyz, along=True)
    >>> np.allclose(len_along, expected_lens.cumsum())
    True
    >>> length([])
    0
    >>> length([[1, 2, 3]])
    0
    >>> length([], along=True)
    array([0])
    """
    xyz = np.asarray(xyz)
    if xyz.shape[0] < 2:
        if along:
            return np.array([0])
        return 0
    dists = np.sqrt((np.diff(xyz, axis=0) ** 2).sum(axis=1))
    if along:
        return np.cumsum(dists)
    return np.sum(dists)


def get_rois_crossed(pointsmm, roiData, voxelSize):
    n_points = len(pointsmm)
    rois_crossed = []
    for j in range(0, n_points):
        # store point
        x = int(pointsmm[j, 0] / float(voxelSize[0]))
        y = int(pointsmm[j, 1] / float(voxelSize[1]))
        z = int(pointsmm[j, 2] / float(voxelSize[2]))
        if not roiData[x, y, z] == 0:
            rois_crossed.append(roiData[x, y, z])
    rois_crossed = list(
        dict.fromkeys(rois_crossed).keys()
    )  # Removed duplicates from the list
    return rois_crossed


def get_connectivity_matrix(n_rois, list_of_roi_crossed_lists):
    connectivity_matrix = np.zeros((n_rois, n_rois), dtype=np.uint)
    for rois_crossed in list_of_roi_crossed_lists:
        for idx_i, roi_i in enumerate(rois_crossed):
            for idx_j, roi_j in enumerate(rois_crossed):
                if idx_i > idx_j:
                    if not roi_i == roi_j:
                        connectivity_matrix[roi_i - 1, roi_j - 1] += 1
    connectivity_matrix = connectivity_matrix + connectivity_matrix.T
    return connectivity_matrix


def create_allpoints_cmat(streamlines, roiData, voxelSize, n_rois):
    """Create the intersection arrays for each fiber"""
    n_fib = len(streamlines)
    pc = -1
    # Computation for each fiber
    final_fiber_ids = []
    list_of_roi_crossed_lists = []
    for i, fiber in enumerate(streamlines):
        pcN = int(round(float(100 * i) / n_fib))
        if pcN > pc and pcN % 1 == 0:
            pc = pcN
            print("%4.0f%%" % (pc))
        rois_crossed = get_rois_crossed(fiber[0], roiData, voxelSize)
        if len(rois_crossed) > 0:
            list_of_roi_crossed_lists.append(list(rois_crossed))
            final_fiber_ids.append(i)

    connectivity_matrix = get_connectivity_matrix(n_rois, list_of_roi_crossed_lists)
    dis = n_fib - len(final_fiber_ids)
    iflogger.info(
        "Found %i (%f percent out of %i fibers) fibers that start or "
        "terminate in a voxel which is not labeled. (orphans)",
        dis,
        dis * 100.0 / n_fib,
        n_fib,
    )
    iflogger.info(
        "Valid fibers: %i (%f percent)", n_fib - dis, 100 - dis * 100.0 / n_fib
    )
    iflogger.info("Returning the intersecting point connectivity matrix")
    return connectivity_matrix, final_fiber_ids


def create_endpoints_array(fib, voxelSize):
    """Create the endpoints arrays for each fiber.

    Parameters
    ----------
    fib : array-like
      the fibers data
    voxelSize : tuple
      3-tuple containing the voxel size of the ROI image

    Returns
    -------
    endpoints : ndarray of size [#fibers, 2, 3]
      containing for each fiber the index of its first and last point in the voxelSize volume
    endpointsmm : ndarray of size [#fibers, 2, 3]
      endpoints in millimeter coordinates

    """
    # Init
    n = len(fib)
    endpoints = np.zeros((n, 2, 3))
    endpointsmm = np.zeros((n, 2, 3))

    # Computation for each fiber
    for i, fi in enumerate(fib):
        f = fi[0]

        # store startpoint
        endpoints[i, 0, :] = f[0, :]
        # store endpoint
        endpoints[i, 1, :] = f[-1, :]

        # store startpoint
        endpointsmm[i, 0, :] = f[0, :]
        # store endpoint
        endpointsmm[i, 1, :] = f[-1, :]

        # Translate from mm to index
        endpoints[i, 0, 0] = int(endpoints[i, 0, 0] / float(voxelSize[0]))
        endpoints[i, 0, 1] = int(endpoints[i, 0, 1] / float(voxelSize[1]))
        endpoints[i, 0, 2] = int(endpoints[i, 0, 2] / float(voxelSize[2]))
        endpoints[i, 1, 0] = int(endpoints[i, 1, 0] / float(voxelSize[0]))
        endpoints[i, 1, 1] = int(endpoints[i, 1, 1] / float(voxelSize[1]))
        endpoints[i, 1, 2] = int(endpoints[i, 1, 2] / float(voxelSize[2]))

    # Return the matrices
    iflogger.info("Returning the endpoint matrix")
    return (endpoints, endpointsmm)


def cmat(
    track_file,
    roi_file,
    resolution_network_file,
    matrix_name,
    matrix_mat_name,
    endpoint_name,
    intersections=False,
):
    """Create the connection matrix for each resolution using fibers and ROIs."""
    import scipy.io as sio

    stats = {}
    iflogger.info("Running cmat function")
    # Identify the endpoints of each fiber
    en_fname = op.abspath(endpoint_name + "_endpoints.npy")
    en_fnamemm = op.abspath(endpoint_name + "_endpointsmm.npy")

    iflogger.info("Reading Trackvis file %s", track_file)
    fib, hdr = nb.trackvis.read(track_file, False)
    stats["orig_n_fib"] = len(fib)

    roi = nb.load(roi_file)
    # Preserve on-disk type unless scaled
    roiData = np.asanyarray(roi.dataobj)
    roiVoxelSize = roi.header.get_zooms()
    (endpoints, endpointsmm) = create_endpoints_array(fib, roiVoxelSize)

    # Output endpoint arrays
    iflogger.info("Saving endpoint array: %s", en_fname)
    np.save(en_fname, endpoints)
    iflogger.info("Saving endpoint array in mm: %s", en_fnamemm)
    np.save(en_fnamemm, endpointsmm)

    n = len(fib)
    iflogger.info("Number of fibers: %i", n)

    # Create empty fiber label array
    fiberlabels = np.zeros((n, 2))
    final_fiberlabels = []
    final_fibers_idx = []

    # Add node information from specified parcellation scheme
    path, name, ext = split_filename(resolution_network_file)
    if ext == ".pck":
        gp = nx.read_gpickle(resolution_network_file)
    elif ext == ".graphml":
        gp = nx.read_graphml(resolution_network_file)
    else:
        raise TypeError("Unable to read file:", resolution_network_file)
    nROIs = len(gp.nodes())

    # add node information from parcellation
    if "dn_position" in gp.nodes[list(gp.nodes())[0]]:
        G = gp.copy()
    else:
        G = nx.Graph()
        for u, d in gp.nodes(data=True):
            G.add_node(int(u), **d)
            # compute a position for the node based on the mean position of the
            # ROI in voxel coordinates (segmentation volume )
            xyz = tuple(
                np.mean(
                    np.where(np.flipud(roiData) == int(d["dn_correspondence_id"])),
                    axis=1,
                )
            )
            G.nodes[int(u)]["dn_position"] = tuple([xyz[0], xyz[2], -xyz[1]])

    if intersections:
        iflogger.info("Filtering tractography from intersections")
        intersection_matrix, final_fiber_ids = create_allpoints_cmat(
            fib, roiData, roiVoxelSize, nROIs
        )
        finalfibers_fname = op.abspath(
            endpoint_name + "_intersections_streamline_final.trk"
        )
        stats["intersections_n_fib"] = save_fibers(
            hdr, fib, finalfibers_fname, final_fiber_ids
        )
        intersection_matrix = np.matrix(intersection_matrix)
        I = G.copy()
        H = nx.from_numpy_matrix(np.matrix(intersection_matrix))
        H = nx.relabel_nodes(H, lambda x: x + 1)  # relabel nodes so they start at 1
        I.add_weighted_edges_from(
            ((u, v, d["weight"]) for u, v, d in H.edges(data=True))
        )

    dis = 0
    for i in range(endpoints.shape[0]):

        # ROI start => ROI end
        try:
            startROI = int(
                roiData[endpoints[i, 0, 0], endpoints[i, 0, 1], endpoints[i, 0, 2]]
            )
            endROI = int(
                roiData[endpoints[i, 1, 0], endpoints[i, 1, 1], endpoints[i, 1, 2]]
            )
        except IndexError:
            iflogger.error(
                "AN INDEXERROR EXCEPTION OCCURED FOR FIBER %s. "
                "PLEASE CHECK ENDPOINT GENERATION",
                i,
            )
            break

        # Filter
        if startROI == 0 or endROI == 0:
            dis += 1
            fiberlabels[i, 0] = -1
            continue

        if startROI > nROIs or endROI > nROIs:
            iflogger.error(
                "Start or endpoint of fiber terminate in a voxel which is labeled higher"
            )
            iflogger.error("than is expected by the parcellation node information.")
            iflogger.error("Start ROI: %i, End ROI: %i", startROI, endROI)
            iflogger.error("This needs bugfixing!")
            continue

        # Update fiber label
        # switch the rois in order to enforce startROI < endROI
        if endROI < startROI:
            tmp = startROI
            startROI = endROI
            endROI = tmp

        fiberlabels[i, 0] = startROI
        fiberlabels[i, 1] = endROI

        final_fiberlabels.append([startROI, endROI])
        final_fibers_idx.append(i)

        # Add edge to graph
        if G.has_edge(startROI, endROI) and "fiblist" in G.edge[startROI][endROI]:
            G.edge[startROI][endROI]["fiblist"].append(i)
        else:
            G.add_edge(startROI, endROI, fiblist=[i])

    # create a final fiber length array
    finalfiberlength = []
    if intersections:
        final_fibers_indices = final_fiber_ids
    else:
        final_fibers_indices = final_fibers_idx

    for idx in final_fibers_indices:
        # compute length of fiber
        finalfiberlength.append(length(fib[idx][0]))

    # convert to array
    final_fiberlength_array = np.array(finalfiberlength)

    # make final fiber labels as array
    final_fiberlabels_array = np.array(final_fiberlabels, dtype=int)

    iflogger.info(
        "Found %i (%f percent out of %i fibers) fibers that start or "
        "terminate in a voxel which is not labeled. (orphans)",
        dis,
        dis * 100.0 / n,
        n,
    )
    iflogger.info("Valid fibers: %i (%f%%)", n - dis, 100 - dis * 100.0 / n)

    numfib = nx.Graph()
    numfib.add_nodes_from(G)
    fibmean = numfib.copy()
    fibmedian = numfib.copy()
    fibdev = numfib.copy()
    for u, v, d in G.edges(data=True):
        G.remove_edge(u, v)
        di = {}
        if "fiblist" in d:
            di["number_of_fibers"] = len(d["fiblist"])
            idx = np.where(
                (final_fiberlabels_array[:, 0] == int(u))
                & (final_fiberlabels_array[:, 1] == int(v))
            )[0]
            di["fiber_length_mean"] = float(np.mean(final_fiberlength_array[idx]))
            di["fiber_length_median"] = float(np.median(final_fiberlength_array[idx]))
            di["fiber_length_std"] = float(np.std(final_fiberlength_array[idx]))
        else:
            di["number_of_fibers"] = 0
            di["fiber_length_mean"] = 0
            di["fiber_length_median"] = 0
            di["fiber_length_std"] = 0
        if not u == v:  # Fix for self loop problem
            G.add_edge(u, v, **di)
            if "fiblist" in d:
                numfib.add_edge(u, v, weight=di["number_of_fibers"])
                fibmean.add_edge(u, v, weight=di["fiber_length_mean"])
                fibmedian.add_edge(u, v, weight=di["fiber_length_median"])
                fibdev.add_edge(u, v, weight=di["fiber_length_std"])

    iflogger.info("Writing network as %s", matrix_name)
    nx.write_gpickle(G, op.abspath(matrix_name))

    numfib_mlab = nx.to_numpy_matrix(numfib, dtype=int)
    numfib_dict = {"number_of_fibers": numfib_mlab}
    fibmean_mlab = nx.to_numpy_matrix(fibmean, dtype=np.float64)
    fibmean_dict = {"mean_fiber_length": fibmean_mlab}
    fibmedian_mlab = nx.to_numpy_matrix(fibmedian, dtype=np.float64)
    fibmedian_dict = {"median_fiber_length": fibmedian_mlab}
    fibdev_mlab = nx.to_numpy_matrix(fibdev, dtype=np.float64)
    fibdev_dict = {"fiber_length_std": fibdev_mlab}

    if intersections:
        path, name, ext = split_filename(matrix_name)
        intersection_matrix_name = op.abspath(name + "_intersections") + ext
        iflogger.info("Writing intersection network as %s", intersection_matrix_name)
        nx.write_gpickle(I, intersection_matrix_name)

    path, name, ext = split_filename(matrix_mat_name)
    if not ext == ".mat":
        ext = ".mat"
        matrix_mat_name = matrix_mat_name + ext

    iflogger.info("Writing matlab matrix as %s", matrix_mat_name)
    sio.savemat(matrix_mat_name, numfib_dict)

    if intersections:
        intersect_dict = {"intersections": intersection_matrix}
        intersection_matrix_mat_name = op.abspath(name + "_intersections") + ext
        iflogger.info("Writing intersection matrix as %s", intersection_matrix_mat_name)
        sio.savemat(intersection_matrix_mat_name, intersect_dict)

    mean_fiber_length_matrix_name = op.abspath(name + "_mean_fiber_length") + ext
    iflogger.info(
        "Writing matlab mean fiber length matrix as %s", mean_fiber_length_matrix_name
    )
    sio.savemat(mean_fiber_length_matrix_name, fibmean_dict)

    median_fiber_length_matrix_name = op.abspath(name + "_median_fiber_length") + ext
    iflogger.info(
        "Writing matlab median fiber length matrix as %s",
        median_fiber_length_matrix_name,
    )
    sio.savemat(median_fiber_length_matrix_name, fibmedian_dict)

    fiber_length_std_matrix_name = op.abspath(name + "_fiber_length_std") + ext
    iflogger.info(
        "Writing matlab fiber length deviation matrix as %s",
        fiber_length_std_matrix_name,
    )
    sio.savemat(fiber_length_std_matrix_name, fibdev_dict)

    fiberlengths_fname = op.abspath(endpoint_name + "_final_fiberslength.npy")
    iflogger.info("Storing final fiber length array as %s", fiberlengths_fname)
    np.save(fiberlengths_fname, final_fiberlength_array)

    fiberlabels_fname = op.abspath(endpoint_name + "_filtered_fiberslabel.npy")
    iflogger.info("Storing all fiber labels (with orphans) as %s", fiberlabels_fname)
    np.save(fiberlabels_fname, np.array(fiberlabels, dtype=np.int32))

    fiberlabels_noorphans_fname = op.abspath(endpoint_name + "_final_fiberslabels.npy")
    iflogger.info(
        "Storing final fiber labels (no orphans) as %s", fiberlabels_noorphans_fname
    )
    np.save(fiberlabels_noorphans_fname, final_fiberlabels_array)

    iflogger.info("Filtering tractography - keeping only no orphan fibers")
    finalfibers_fname = op.abspath(endpoint_name + "_streamline_final.trk")
    stats["endpoint_n_fib"] = save_fibers(hdr, fib, finalfibers_fname, final_fibers_idx)
    stats["endpoints_percent"] = (
        float(stats["endpoint_n_fib"]) / float(stats["orig_n_fib"]) * 100
    )
    stats["intersections_percent"] = (
        float(stats["intersections_n_fib"]) / float(stats["orig_n_fib"]) * 100
    )

    out_stats_file = op.abspath(endpoint_name + "_statistics.mat")
    iflogger.info("Saving matrix creation statistics as %s", out_stats_file)
    sio.savemat(out_stats_file, stats)


def save_fibers(oldhdr, oldfib, fname, indices):
    """Stores a new trackvis file fname using only given indices"""
    hdrnew = oldhdr.copy()
    outstreams = []
    for i in indices:
        outstreams.append(oldfib[i])
    n_fib_out = len(outstreams)
    hdrnew["n_count"] = n_fib_out
    iflogger.info("Writing final non-orphan fibers as %s", fname)
    nb.trackvis.write(fname, outstreams, hdrnew)
    return n_fib_out


class CreateMatrixInputSpec(TraitedSpec):
    roi_file = File(exists=True, mandatory=True, desc="Freesurfer aparc+aseg file")
    tract_file = File(exists=True, mandatory=True, desc="Trackvis tract file")
    resolution_network_file = File(
        exists=True,
        mandatory=True,
        desc="Parcellation files from Connectome Mapping Toolkit",
    )
    count_region_intersections = traits.Bool(
        False,
        usedefault=True,
        desc="Counts all of the fiber-region traversals in the connectivity matrix (requires significantly more computational time)",
    )
    out_matrix_file = File(
        genfile=True, desc="NetworkX graph describing the connectivity"
    )
    out_matrix_mat_file = File(
        "cmatrix.mat", usedefault=True, desc="Matlab matrix describing the connectivity"
    )
    out_mean_fiber_length_matrix_mat_file = File(
        genfile=True,
        desc="Matlab matrix describing the mean fiber lengths between each node.",
    )
    out_median_fiber_length_matrix_mat_file = File(
        genfile=True,
        desc="Matlab matrix describing the mean fiber lengths between each node.",
    )
    out_fiber_length_std_matrix_mat_file = File(
        genfile=True,
        desc="Matlab matrix describing the deviation in fiber lengths connecting each node.",
    )
    out_intersection_matrix_mat_file = File(
        genfile=True,
        desc="Matlab connectivity matrix if all region/fiber intersections are counted.",
    )
    out_endpoint_array_name = File(
        genfile=True, desc="Name for the generated endpoint arrays"
    )


class CreateMatrixOutputSpec(TraitedSpec):
    matrix_file = File(desc="NetworkX graph describing the connectivity", exists=True)
    intersection_matrix_file = File(
        desc="NetworkX graph describing the connectivity", exists=True
    )
    matrix_files = OutputMultiPath(
        File(
            desc="All of the gpickled network files output by this interface",
            exists=True,
        )
    )
    matlab_matrix_files = OutputMultiPath(
        File(desc="All of the MATLAB .mat files output by this interface", exists=True)
    )
    matrix_mat_file = File(
        desc="Matlab matrix describing the connectivity", exists=True
    )
    intersection_matrix_mat_file = File(
        desc="Matlab matrix describing the mean fiber lengths between each node.",
        exists=True,
    )
    mean_fiber_length_matrix_mat_file = File(
        desc="Matlab matrix describing the mean fiber lengths between each node.",
        exists=True,
    )
    median_fiber_length_matrix_mat_file = File(
        desc="Matlab matrix describing the median fiber lengths between each node.",
        exists=True,
    )
    fiber_length_std_matrix_mat_file = File(
        desc="Matlab matrix describing the deviation in fiber lengths connecting each node.",
        exists=True,
    )
    endpoint_file = File(
        desc="Saved Numpy array with the endpoints of each fiber", exists=True
    )
    endpoint_file_mm = File(
        desc="Saved Numpy array with the endpoints of each fiber (in millimeters)",
        exists=True,
    )
    fiber_length_file = File(
        desc="Saved Numpy array with the lengths of each fiber", exists=True
    )
    fiber_label_file = File(
        desc="Saved Numpy array with the labels for each fiber", exists=True
    )
    fiber_labels_noorphans = File(
        desc="Saved Numpy array with the labels for each non-orphan fiber", exists=True
    )
    filtered_tractography = File(
        desc="TrackVis file containing only those fibers originate in one and terminate in another region",
        exists=True,
    )
    filtered_tractography_by_intersections = File(
        desc="TrackVis file containing all fibers which connect two regions",
        exists=True,
    )
    filtered_tractographies = OutputMultiPath(
        File(
            desc="TrackVis file containing only those fibers originate in one and terminate in another region",
            exists=True,
        )
    )
    stats_file = File(
        desc="Saved Matlab .mat file with the number of fibers saved at each stage",
        exists=True,
    )


class CreateMatrix(BaseInterface):
    """
    Performs connectivity mapping and outputs the result as a NetworkX graph and a Matlab matrix

    Example
    -------

    >>> import nipype.interfaces.cmtk as cmtk
    >>> conmap = cmtk.CreateMatrix()
    >>> conmap.roi_file = 'fsLUT_aparc+aseg.nii'
    >>> conmap.tract_file = 'fibers.trk'
    >>> conmap.run()                 # doctest: +SKIP
    """

    input_spec = CreateMatrixInputSpec
    output_spec = CreateMatrixOutputSpec

    def _run_interface(self, runtime):
        if isdefined(self.inputs.out_matrix_file):
            path, name, _ = split_filename(self.inputs.out_matrix_file)
            matrix_file = op.abspath(name + ".pck")
        else:
            matrix_file = self._gen_outfilename(".pck")

        matrix_mat_file = op.abspath(self.inputs.out_matrix_mat_file)
        path, name, ext = split_filename(matrix_mat_file)
        if not ext == ".mat":
            ext = ".mat"
            matrix_mat_file = matrix_mat_file + ext

        if isdefined(self.inputs.out_mean_fiber_length_matrix_mat_file):
            mean_fiber_length_matrix_mat_file = op.abspath(
                self.inputs.out_mean_fiber_length_matrix_mat_file
            )
        else:
            mean_fiber_length_matrix_name = op.abspath(
                self._gen_outfilename("_mean_fiber_length.mat")
            )

        if isdefined(self.inputs.out_median_fiber_length_matrix_mat_file):
            median_fiber_length_matrix_mat_file = op.abspath(
                self.inputs.out_median_fiber_length_matrix_mat_file
            )
        else:
            median_fiber_length_matrix_name = op.abspath(
                self._gen_outfilename("_median_fiber_length.mat")
            )

        if isdefined(self.inputs.out_fiber_length_std_matrix_mat_file):
            fiber_length_std_matrix_mat_file = op.abspath(
                self.inputs.out_fiber_length_std_matrix_mat_file
            )
        else:
            fiber_length_std_matrix_name = op.abspath(
                self._gen_outfilename("_fiber_length_std.mat")
            )

        if not isdefined(self.inputs.out_endpoint_array_name):
            _, endpoint_name, _ = split_filename(self.inputs.tract_file)
            endpoint_name = op.abspath(endpoint_name)
        else:
            endpoint_name = op.abspath(self.inputs.out_endpoint_array_name)

        cmat(
            self.inputs.tract_file,
            self.inputs.roi_file,
            self.inputs.resolution_network_file,
            matrix_file,
            matrix_mat_file,
            endpoint_name,
            self.inputs.count_region_intersections,
        )
        return runtime

    def _list_outputs(self):
        outputs = self.output_spec().get()
        if isdefined(self.inputs.out_matrix_file):
            path, name, _ = split_filename(self.inputs.out_matrix_file)
            out_matrix_file = op.abspath(name + ".pck")
            out_intersection_matrix_file = op.abspath(name + "_intersections.pck")
        else:
            out_matrix_file = op.abspath(self._gen_outfilename(".pck"))
            out_intersection_matrix_file = op.abspath(
                self._gen_outfilename("_intersections.pck")
            )

        outputs["matrix_file"] = out_matrix_file
        outputs["intersection_matrix_file"] = out_intersection_matrix_file

        matrix_mat_file = op.abspath(self.inputs.out_matrix_mat_file)
        path, name, ext = split_filename(matrix_mat_file)
        if not ext == ".mat":
            ext = ".mat"
            matrix_mat_file = matrix_mat_file + ext

        outputs["matrix_mat_file"] = matrix_mat_file
        if isdefined(self.inputs.out_mean_fiber_length_matrix_mat_file):
            outputs["mean_fiber_length_matrix_mat_file"] = op.abspath(
                self.inputs.out_mean_fiber_length_matrix_mat_file
            )
        else:
            outputs["mean_fiber_length_matrix_mat_file"] = op.abspath(
                self._gen_outfilename("_mean_fiber_length.mat")
            )

        if isdefined(self.inputs.out_median_fiber_length_matrix_mat_file):
            outputs["median_fiber_length_matrix_mat_file"] = op.abspath(
                self.inputs.out_median_fiber_length_matrix_mat_file
            )
        else:
            outputs["median_fiber_length_matrix_mat_file"] = op.abspath(
                self._gen_outfilename("_median_fiber_length.mat")
            )

        if isdefined(self.inputs.out_fiber_length_std_matrix_mat_file):
            outputs["fiber_length_std_matrix_mat_file"] = op.abspath(
                self.inputs.out_fiber_length_std_matrix_mat_file
            )
        else:
            outputs["fiber_length_std_matrix_mat_file"] = op.abspath(
                self._gen_outfilename("_fiber_length_std.mat")
            )

        if isdefined(self.inputs.out_intersection_matrix_mat_file):
            outputs["intersection_matrix_mat_file"] = op.abspath(
                self.inputs.out_intersection_matrix_mat_file
            )
        else:
            outputs["intersection_matrix_mat_file"] = op.abspath(
                self._gen_outfilename("_intersections.mat")
            )

        if isdefined(self.inputs.out_endpoint_array_name):
            endpoint_name = self.inputs.out_endpoint_array_name
            outputs["endpoint_file"] = op.abspath(
                self.inputs.out_endpoint_array_name + "_endpoints.npy"
            )
            outputs["endpoint_file_mm"] = op.abspath(
                self.inputs.out_endpoint_array_name + "_endpointsmm.npy"
            )
            outputs["fiber_length_file"] = op.abspath(
                self.inputs.out_endpoint_array_name + "_final_fiberslength.npy"
            )
            outputs["fiber_label_file"] = op.abspath(
                self.inputs.out_endpoint_array_name + "_filtered_fiberslabel.npy"
            )
            outputs["fiber_labels_noorphans"] = op.abspath(
                self.inputs.out_endpoint_array_name + "_final_fiberslabels.npy"
            )
        else:
            _, endpoint_name, _ = split_filename(self.inputs.tract_file)
            outputs["endpoint_file"] = op.abspath(endpoint_name + "_endpoints.npy")
            outputs["endpoint_file_mm"] = op.abspath(endpoint_name + "_endpointsmm.npy")
            outputs["fiber_length_file"] = op.abspath(
                endpoint_name + "_final_fiberslength.npy"
            )
            outputs["fiber_label_file"] = op.abspath(
                endpoint_name + "_filtered_fiberslabel.npy"
            )
            outputs["fiber_labels_noorphans"] = op.abspath(
                endpoint_name + "_final_fiberslabels.npy"
            )

        if self.inputs.count_region_intersections:
            outputs["matrix_files"] = [out_matrix_file, out_intersection_matrix_file]
            outputs["matlab_matrix_files"] = [
                outputs["matrix_mat_file"],
                outputs["mean_fiber_length_matrix_mat_file"],
                outputs["median_fiber_length_matrix_mat_file"],
                outputs["fiber_length_std_matrix_mat_file"],
                outputs["intersection_matrix_mat_file"],
            ]
        else:
            outputs["matrix_files"] = [out_matrix_file]
            outputs["matlab_matrix_files"] = [
                outputs["matrix_mat_file"],
                outputs["mean_fiber_length_matrix_mat_file"],
                outputs["median_fiber_length_matrix_mat_file"],
                outputs["fiber_length_std_matrix_mat_file"],
            ]

        outputs["filtered_tractography"] = op.abspath(
            endpoint_name + "_streamline_final.trk"
        )
        outputs["filtered_tractography_by_intersections"] = op.abspath(
            endpoint_name + "_intersections_streamline_final.trk"
        )
        outputs["filtered_tractographies"] = [
            outputs["filtered_tractography"],
            outputs["filtered_tractography_by_intersections"],
        ]
        outputs["stats_file"] = op.abspath(endpoint_name + "_statistics.mat")
        return outputs

    def _gen_outfilename(self, ext):
        if ext.endswith("mat") and isdefined(self.inputs.out_matrix_mat_file):
            _, name, _ = split_filename(self.inputs.out_matrix_mat_file)
        elif isdefined(self.inputs.out_matrix_file):
            _, name, _ = split_filename(self.inputs.out_matrix_file)
        else:
            _, name, _ = split_filename(self.inputs.tract_file)
        return name + ext


class ROIGenInputSpec(BaseInterfaceInputSpec):
    aparc_aseg_file = File(
        exists=True, mandatory=True, desc="Freesurfer aparc+aseg file"
    )
    LUT_file = File(
        exists=True,
        xor=["use_freesurfer_LUT"],
        desc="Custom lookup table (cf. FreeSurferColorLUT.txt)",
    )
    use_freesurfer_LUT = traits.Bool(
        xor=["LUT_file"],
        desc="Boolean value; Set to True to use default Freesurfer LUT, False for custom LUT",
    )
    freesurfer_dir = Directory(
        requires=["use_freesurfer_LUT"], desc="Freesurfer main directory"
    )
    out_roi_file = File(
        genfile=True, desc="Region of Interest file for connectivity mapping"
    )
    out_dict_file = File(genfile=True, desc="Label dictionary saved in Pickle format")


class ROIGenOutputSpec(TraitedSpec):
    roi_file = File(desc="Region of Interest file for connectivity mapping")
    dict_file = File(desc="Label dictionary saved in Pickle format")


class ROIGen(BaseInterface):
    """
    Generates a ROI file for connectivity mapping and a dictionary file containing relevant node information

    Example
    -------

    >>> import nipype.interfaces.cmtk as cmtk
    >>> rg = cmtk.ROIGen()
    >>> rg.inputs.aparc_aseg_file = 'aparc+aseg.nii'
    >>> rg.inputs.use_freesurfer_LUT = True
    >>> rg.inputs.freesurfer_dir = '/usr/local/freesurfer'
    >>> rg.run() # doctest: +SKIP

    The label dictionary is written to disk using Pickle. Resulting data can be loaded using:

    >>> file = open("FreeSurferColorLUT_adapted_aparc+aseg_out.pck", "r")
    >>> file = open("fsLUT_aparc+aseg.pck", "r")
    >>> labelDict = pickle.load(file) # doctest: +SKIP
    >>> labelDict                     # doctest: +SKIP
    """

    input_spec = ROIGenInputSpec
    output_spec = ROIGenOutputSpec

    def _run_interface(self, runtime):
        aparc_aseg_file = self.inputs.aparc_aseg_file
        aparcpath, aparcname, aparcext = split_filename(aparc_aseg_file)
        iflogger.info("Using Aparc+Aseg file: %s", aparcname + aparcext)
        niiAPARCimg = nb.load(aparc_aseg_file)
        # Preserve on-disk type
        niiAPARCdata = np.asanyarray(niiAPARCimg.dataobj)
        niiDataLabels = np.unique(niiAPARCdata)
        numDataLabels = np.size(niiDataLabels)
        iflogger.info("Number of labels in image: %s", numDataLabels)

        write_dict = True
        if self.inputs.use_freesurfer_LUT:
            self.LUT_file = self.inputs.freesurfer_dir + "/FreeSurferColorLUT.txt"
            iflogger.info("Using Freesurfer LUT: %s", self.LUT_file)
            prefix = "fsLUT"
        elif not self.inputs.use_freesurfer_LUT and isdefined(self.inputs.LUT_file):
            self.LUT_file = op.abspath(self.inputs.LUT_file)
            lutpath, lutname, lutext = split_filename(self.LUT_file)
            iflogger.info("Using Custom LUT file: %s", lutname + lutext)
            prefix = lutname
        else:
            prefix = "hardcoded"
            write_dict = False

        if isdefined(self.inputs.out_roi_file):
            roi_file = op.abspath(self.inputs.out_roi_file)
        else:
            roi_file = op.abspath(prefix + "_" + aparcname + ".nii")

        if isdefined(self.inputs.out_dict_file):
            dict_file = op.abspath(self.inputs.out_dict_file)
        else:
            dict_file = op.abspath(prefix + "_" + aparcname + ".pck")

        if write_dict:
            iflogger.info("Lookup table: %s", op.abspath(self.LUT_file))
            LUTlabelsRGBA = np.loadtxt(
                self.LUT_file,
                skiprows=4,
                usecols=[0, 1, 2, 3, 4, 5],
                comments="#",
                dtype={
                    "names": ("index", "label", "R", "G", "B", "A"),
                    "formats": ("int", "|S30", "int", "int", "int", "int"),
                },
            )
            numLUTLabels = np.size(LUTlabelsRGBA)
            if numLUTLabels < numDataLabels:
                iflogger.error(
                    "LUT file provided does not contain all of the regions in the image"
                )
                iflogger.error("Removing unmapped regions")
            iflogger.info("Number of labels in LUT: %s", numLUTLabels)
            LUTlabelDict = {}
            """ Create dictionary for input LUT table"""
            for labels in range(0, numLUTLabels):
                LUTlabelDict[LUTlabelsRGBA[labels][0]] = [
                    LUTlabelsRGBA[labels][1],
                    LUTlabelsRGBA[labels][2],
                    LUTlabelsRGBA[labels][3],
                    LUTlabelsRGBA[labels][4],
                    LUTlabelsRGBA[labels][5],
                ]

            iflogger.info("Printing LUT label dictionary")
            iflogger.info(LUTlabelDict)

        mapDict = {}
        MAPPING = [
            [1, 2012],
            [2, 2019],
            [3, 2032],
            [4, 2014],
            [5, 2020],
            [6, 2018],
            [7, 2027],
            [8, 2028],
            [9, 2003],
            [10, 2024],
            [11, 2017],
            [12, 2026],
            [13, 2002],
            [14, 2023],
            [15, 2010],
            [16, 2022],
            [17, 2031],
            [18, 2029],
            [19, 2008],
            [20, 2025],
            [21, 2005],
            [22, 2021],
            [23, 2011],
            [24, 2013],
            [25, 2007],
            [26, 2016],
            [27, 2006],
            [28, 2033],
            [29, 2009],
            [30, 2015],
            [31, 2001],
            [32, 2030],
            [33, 2034],
            [34, 2035],
            [35, 49],
            [36, 50],
            [37, 51],
            [38, 52],
            [39, 58],
            [40, 53],
            [41, 54],
            [42, 1012],
            [43, 1019],
            [44, 1032],
            [45, 1014],
            [46, 1020],
            [47, 1018],
            [48, 1027],
            [49, 1028],
            [50, 1003],
            [51, 1024],
            [52, 1017],
            [53, 1026],
            [54, 1002],
            [55, 1023],
            [56, 1010],
            [57, 1022],
            [58, 1031],
            [59, 1029],
            [60, 1008],
            [61, 1025],
            [62, 1005],
            [63, 1021],
            [64, 1011],
            [65, 1013],
            [66, 1007],
            [67, 1016],
            [68, 1006],
            [69, 1033],
            [70, 1009],
            [71, 1015],
            [72, 1001],
            [73, 1030],
            [74, 1034],
            [75, 1035],
            [76, 10],
            [77, 11],
            [78, 12],
            [79, 13],
            [80, 26],
            [81, 17],
            [82, 18],
            [83, 16],
        ]
        """ Create empty grey matter mask, Populate with only those regions defined in the mapping."""
        niiGM = np.zeros(niiAPARCdata.shape, dtype=np.uint)
        for ma in MAPPING:
            niiGM[niiAPARCdata == ma[1]] = ma[0]
            mapDict[ma[0]] = ma[1]
        iflogger.info("Grey matter mask created")
        greyMaskLabels = np.unique(niiGM)
        numGMLabels = np.size(greyMaskLabels)
        iflogger.info("Number of grey matter labels: %s", numGMLabels)

        labelDict = {}
        GMlabelDict = {}
        for label in greyMaskLabels:
            try:
                mapDict[label]
                if write_dict:
                    GMlabelDict["originalID"] = mapDict[label]
            except:
                iflogger.info("Label %s not in provided mapping", label)
            if write_dict:
                del GMlabelDict
                GMlabelDict = {}
                GMlabelDict["labels"] = LUTlabelDict[label][0]
                GMlabelDict["colors"] = [
                    LUTlabelDict[label][1],
                    LUTlabelDict[label][2],
                    LUTlabelDict[label][3],
                ]
                GMlabelDict["a"] = LUTlabelDict[label][4]
                labelDict[label] = GMlabelDict

        roi_image = nb.Nifti1Image(niiGM, niiAPARCimg.affine, niiAPARCimg.header)
        iflogger.info("Saving ROI File to %s", roi_file)
        nb.save(roi_image, roi_file)

        if write_dict:
            iflogger.info("Saving Dictionary File to %s in Pickle format", dict_file)
            with open(dict_file, "w") as f:
                pickle.dump(labelDict, f)
        return runtime

    def _list_outputs(self):
        outputs = self._outputs().get()
        if isdefined(self.inputs.out_roi_file):
            outputs["roi_file"] = op.abspath(self.inputs.out_roi_file)
        else:
            outputs["roi_file"] = op.abspath(self._gen_outfilename("nii"))
        if isdefined(self.inputs.out_dict_file):
            outputs["dict_file"] = op.abspath(self.inputs.out_dict_file)
        else:
            outputs["dict_file"] = op.abspath(self._gen_outfilename("pck"))
        return outputs

    def _gen_outfilename(self, ext):
        _, name, _ = split_filename(self.inputs.aparc_aseg_file)
        if self.inputs.use_freesurfer_LUT:
            prefix = "fsLUT"
        elif not self.inputs.use_freesurfer_LUT and isdefined(self.inputs.LUT_file):
            lutpath, lutname, lutext = split_filename(self.inputs.LUT_file)
            prefix = lutname
        else:
            prefix = "hardcoded"
        return prefix + "_" + name + "." + ext


def create_nodes(roi_file, resolution_network_file, out_filename):
    G = nx.Graph()
    gp = nx.read_graphml(resolution_network_file)
    roi_image = nb.load(roi_file)
    # Preserve on-disk type unless scaled
    roiData = np.asanyarray(roi_image.dataobj)
    for u, d in gp.nodes(data=True):
        G.add_node(int(u), **d)
        xyz = tuple(
            np.mean(
                np.where(np.flipud(roiData) == int(d["dn_correspondence_id"])), axis=1
            )
        )
        G.nodes[int(u)]["dn_position"] = tuple([xyz[0], xyz[2], -xyz[1]])
    nx.write_gpickle(G, out_filename)
    return out_filename


class CreateNodesInputSpec(BaseInterfaceInputSpec):
    roi_file = File(exists=True, mandatory=True, desc="Region of interest file")
    resolution_network_file = File(
        exists=True,
        mandatory=True,
        desc="Parcellation file from Connectome Mapping Toolkit",
    )
    out_filename = File(
        "nodenetwork.pck",
        usedefault=True,
        desc="Output gpickled network with the nodes defined.",
    )


class CreateNodesOutputSpec(TraitedSpec):
    node_network = File(desc="Output gpickled network with the nodes defined.")


class CreateNodes(BaseInterface):
    """
    Generates a NetworkX graph containing nodes at the centroid of each region in the input ROI file.
    Node data is added from the resolution network file.

    Example
    -------

    >>> import nipype.interfaces.cmtk as cmtk
    >>> mknode = cmtk.CreateNodes()
    >>> mknode.inputs.roi_file = 'ROI_scale500.nii.gz'
    >>> mknode.run() # doctest: +SKIP
    """

    input_spec = CreateNodesInputSpec
    output_spec = CreateNodesOutputSpec

    def _run_interface(self, runtime):
        iflogger.info("Creating nodes...")
        create_nodes(
            self.inputs.roi_file,
            self.inputs.resolution_network_file,
            self.inputs.out_filename,
        )
        iflogger.info("Saving node network to %s", op.abspath(self.inputs.out_filename))
        return runtime

    def _list_outputs(self):
        outputs = self._outputs().get()
        outputs["node_network"] = op.abspath(self.inputs.out_filename)
        return outputs



# -*- coding: utf-8 -*-
"""CMP implements a full processing pipeline for creating connectomes with dMRI data."""
from .cmtk import ROIGen, CreateMatrix, CreateNodes
from .nx import NetworkXMetrics, AverageNetworks
from .parcellation import Parcellate
from .convert import CFFConverter, MergeCNetworks
from .nbs import NetworkBasedStatistic



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..preprocess import MRIConvert


def test_MRIConvert_inputs():
    input_map = dict(
        apply_inv_transform=dict(
            argstr="--apply_inverse_transform %s",
            extensions=None,
        ),
        apply_transform=dict(
            argstr="--apply_transform %s",
            extensions=None,
        ),
        args=dict(
            argstr="%s",
        ),
        ascii=dict(
            argstr="--ascii",
        ),
        autoalign_matrix=dict(
            argstr="--autoalign %s",
            extensions=None,
        ),
        color_file=dict(
            argstr="--color_file %s",
            extensions=None,
        ),
        conform=dict(
            argstr="--conform",
        ),
        conform_min=dict(
            argstr="--conform_min",
        ),
        conform_size=dict(
            argstr="--conform_size %s",
        ),
        crop_center=dict(
            argstr="--crop %d %d %d",
        ),
        crop_gdf=dict(
            argstr="--crop_gdf",
        ),
        crop_size=dict(
            argstr="--cropsize %d %d %d",
        ),
        cut_ends=dict(
            argstr="--cutends %d",
        ),
        cw256=dict(
            argstr="--cw256",
        ),
        devolve_transform=dict(
            argstr="--devolvexfm %s",
        ),
        drop_n=dict(
            argstr="--ndrop %d",
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        fill_parcellation=dict(
            argstr="--fill_parcellation",
        ),
        force_ras=dict(
            argstr="--force_ras_good",
        ),
        frame=dict(
            argstr="--frame %d",
        ),
        frame_subsample=dict(
            argstr="--fsubsample %d %d %d",
        ),
        fwhm=dict(
            argstr="--fwhm %f",
        ),
        in_center=dict(
            argstr="--in_center %s",
        ),
        in_file=dict(
            argstr="--input_volume %s",
            extensions=None,
            mandatory=True,
            position=-2,
        ),
        in_i_dir=dict(
            argstr="--in_i_direction %f %f %f",
        ),
        in_i_size=dict(
            argstr="--in_i_size %d",
        ),
        in_info=dict(
            argstr="--in_info",
        ),
        in_j_dir=dict(
            argstr="--in_j_direction %f %f %f",
        ),
        in_j_size=dict(
            argstr="--in_j_size %d",
        ),
        in_k_dir=dict(
            argstr="--in_k_direction %f %f %f",
        ),
        in_k_size=dict(
            argstr="--in_k_size %d",
        ),
        in_like=dict(
            argstr="--in_like %s",
            extensions=None,
        ),
        in_matrix=dict(
            argstr="--in_matrix",
        ),
        in_orientation=dict(
            argstr="--in_orientation %s",
        ),
        in_scale=dict(
            argstr="--scale %f",
        ),
        in_stats=dict(
            argstr="--in_stats",
        ),
        in_type=dict(
            argstr="--in_type %s",
        ),
        invert_contrast=dict(
            argstr="--invert_contrast %f",
        ),
        midframe=dict(
            argstr="--mid-frame",
        ),
        no_change=dict(
            argstr="--nochange",
        ),
        no_scale=dict(
            argstr="--no_scale 1",
        ),
        no_translate=dict(
            argstr="--no_translate",
        ),
        no_write=dict(
            argstr="--no_write",
        ),
        out_center=dict(
            argstr="--out_center %f %f %f",
        ),
        out_datatype=dict(
            argstr="--out_data_type %s",
        ),
        out_file=dict(
            argstr="--output_volume %s",
            extensions=None,
            genfile=True,
            position=-1,
        ),
        out_i_count=dict(
            argstr="--out_i_count %d",
        ),
        out_i_dir=dict(
            argstr="--out_i_direction %f %f %f",
        ),
        out_i_size=dict(
            argstr="--out_i_size %d",
        ),
        out_info=dict(
            argstr="--out_info",
        ),
        out_j_count=dict(
            argstr="--out_j_count %d",
        ),
        out_j_dir=dict(
            argstr="--out_j_direction %f %f %f",
        ),
        out_j_size=dict(
            argstr="--out_j_size %d",
        ),
        out_k_count=dict(
            argstr="--out_k_count %d",
        ),
        out_k_dir=dict(
            argstr="--out_k_direction %f %f %f",
        ),
        out_k_size=dict(
            argstr="--out_k_size %d",
        ),
        out_matrix=dict(
            argstr="--out_matrix",
        ),
        out_orientation=dict(
            argstr="--out_orientation %s",
        ),
        out_scale=dict(
            argstr="--out-scale %d",
        ),
        out_stats=dict(
            argstr="--out_stats",
        ),
        out_type=dict(
            argstr="--out_type %s",
        ),
        parse_only=dict(
            argstr="--parse_only",
        ),
        read_only=dict(
            argstr="--read_only",
        ),
        reorder=dict(
            argstr="--reorder %d %d %d",
        ),
        resample_type=dict(
            argstr="--resample_type %s",
        ),
        reslice_like=dict(
            argstr="--reslice_like %s",
            extensions=None,
        ),
        sdcm_list=dict(
            argstr="--sdcmlist %s",
            extensions=None,
        ),
        skip_n=dict(
            argstr="--nskip %d",
        ),
        slice_bias=dict(
            argstr="--slice-bias %f",
        ),
        slice_crop=dict(
            argstr="--slice-crop %d %d",
        ),
        slice_reverse=dict(
            argstr="--slice-reverse",
        ),
        smooth_parcellation=dict(
            argstr="--smooth_parcellation",
        ),
        sphinx=dict(
            argstr="--sphinx",
        ),
        split=dict(
            argstr="--split",
        ),
        status_file=dict(
            argstr="--status %s",
            extensions=None,
        ),
        subject_name=dict(
            argstr="--subject_name %s",
        ),
        subjects_dir=dict(),
        te=dict(
            argstr="-te %d",
        ),
        template_info=dict(
            argstr="--template_info",
        ),
        template_type=dict(
            argstr="--template_type %s",
        ),
        ti=dict(
            argstr="-ti %d",
        ),
        tr=dict(
            argstr="-tr %d",
        ),
        unwarp_gradient=dict(
            argstr="--unwarp_gradient_nonlinearity",
        ),
        vox_size=dict(
            argstr="-voxsize %f %f %f",
        ),
        zero_ge_z_offset=dict(
            argstr="--zero_ge_z_offset",
        ),
        zero_outlines=dict(
            argstr="--zero_outlines",
        ),
    )
    inputs = MRIConvert.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_MRIConvert_outputs():
    output_map = dict(
        out_file=dict(),
    )
    outputs = MRIConvert.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
# -*- coding: utf8 -*-
"""Autogenerated file - DO NOT EDIT
If you spot a bug, please report it on the mailing list and/or change the generator."""

import os

from ...base import (
    CommandLine,
    CommandLineInputSpec,
    SEMLikeCommandLine,
    TraitedSpec,
    File,
    Directory,
    traits,
    isdefined,
    InputMultiPath,
    OutputMultiPath,
)


class UnbiasedNonLocalMeansInputSpec(CommandLineInputSpec):
    sigma = traits.Float(
        desc="The root power of noise (sigma) in the complex Gaussian process the Rician comes from. If it is underestimated, the algorithm fails to remove the noise. If it is overestimated, over-blurring is likely to occur.",
        argstr="--sigma %f",
    )
    rs = InputMultiPath(
        traits.Int,
        desc="The algorithm search for similar voxels in a neighborhood of this radius (radii larger than 5,5,5 are very slow, and the results can be only marginally better. Small radii may fail to effectively remove the noise).",
        sep=",",
        argstr="--rs %s",
    )
    rc = InputMultiPath(
        traits.Int,
        desc="Similarity between blocks is computed as the difference between mean values and gradients. These parameters are computed fitting a hyperplane with LS inside a neighborhood of this size",
        sep=",",
        argstr="--rc %s",
    )
    hp = traits.Float(
        desc="This parameter is related to noise; the larger the parameter, the more aggressive the filtering. Should be near 1, and only values between 0.8 and 1.2 are allowed",
        argstr="--hp %f",
    )
    ps = traits.Float(
        desc="To accelerate computations, preselection is used: if the normalized difference is above this threshold, the voxel will be discarded (non used for average)",
        argstr="--ps %f",
    )
    inputVolume = File(position=-2, desc="Input MRI volume.", exists=True, argstr="%s")
    outputVolume = traits.Either(
        traits.Bool,
        File(),
        position=-1,
        hash_files=False,
        desc="Output (filtered) MRI volume.",
        argstr="%s",
    )


class UnbiasedNonLocalMeansOutputSpec(TraitedSpec):
    outputVolume = File(position=-1, desc="Output (filtered) MRI volume.", exists=True)


class UnbiasedNonLocalMeans(SEMLikeCommandLine):
    """title: Unbiased NLM for MRI

    category: Filtering.Denoising

    description: This module implements a fast version of the popular Non-Local Means filter for image denoising. This algorithm filters each pixel as a weighted average of its neighbors in a large vicinity. The weights are computed based on the similarity of each neighbor with the voxel to be denoised.
     In the original formulation a patch with a certain radius is centered in each of the voxels, and the Mean Squared Error between each pair of corresponding voxels is computed. In this implementation, only the mean value and gradient components are compared. This, together with an efficient memory management, can attain a speed-up of nearly 20x. Besides, the filtering is more accurate than the original with poor SNR.
     This code is intended for its use with MRI (or any other Rician-distributed modality): the second order moment is estimated, then we subtract twice the squared power of noise, and finally we take the square root of the result to remove the Rician bias.
     The original implementation of the NLM filter may be found in:
     A. Buades, B. Coll, J. Morel, "A review of image denoising algorithms, with a new one", Multiscale Modelling and Simulation 4(2): 490-530. 2005.
     The correction of the Rician bias is described in the following reference (among others):
     S. Aja-Fernandez, K. Krissian, "An unbiased Non-Local Means scheme for DWI filtering", in: Proceedings of the MICCAI Workshop on Computational Diffusion MRI, 2008, pp. 277-284.
     The whole description of this version may be found in the following paper (please, cite it if you are willing to use this software):
     A. Tristan-Vega, V. Garcia Perez, S. Aja-Fenandez, and C.-F. Westin, "Efficient and Robust Nonlocal Means Denoising of MR Data Based on Salient Features Matching", Computer Methods and Programs in Biomedicine. (Accepted for publication) 2011.

    version: 0.0.1.$Revision: 1 $(beta)

    documentation-url: http://www.slicer.org/slicerWiki/index.php/Modules:UnbiasedNonLocalMeans-Documentation-3.6

    contributor: Antonio Tristan Vega, Veronica Garcia-Perez, Santiago Aja-Fernandez, Carl-Fredrik Westin

    acknowledgements: Supported by grant number FMECD-2010/71131616E from the Spanish Ministry of Education/Fulbright Committee
    """

    input_spec = UnbiasedNonLocalMeansInputSpec
    output_spec = UnbiasedNonLocalMeansOutputSpec
    _cmd = " UnbiasedNonLocalMeans "
    _outputs_filenames = {"outputVolume": "outputVolume.nii"}
    _redirect_x = False



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..specialized import VBRAINSDemonWarp


def test_VBRAINSDemonWarp_inputs():
    input_map = dict(
        args=dict(
            argstr="%s",
        ),
        arrayOfPyramidLevelIterations=dict(
            argstr="--arrayOfPyramidLevelIterations %s",
            sep=",",
        ),
        backgroundFillValue=dict(
            argstr="--backgroundFillValue %d",
        ),
        checkerboardPatternSubdivisions=dict(
            argstr="--checkerboardPatternSubdivisions %s",
            sep=",",
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        fixedBinaryVolume=dict(
            argstr="--fixedBinaryVolume %s",
            extensions=None,
        ),
        fixedVolume=dict(
            argstr="--fixedVolume %s...",
        ),
        gradient_type=dict(
            argstr="--gradient_type %s",
        ),
        gui=dict(
            argstr="--gui ",
        ),
        histogramMatch=dict(
            argstr="--histogramMatch ",
        ),
        initializeWithDisplacementField=dict(
            argstr="--initializeWithDisplacementField %s",
            extensions=None,
        ),
        initializeWithTransform=dict(
            argstr="--initializeWithTransform %s",
            extensions=None,
        ),
        inputPixelType=dict(
            argstr="--inputPixelType %s",
        ),
        interpolationMode=dict(
            argstr="--interpolationMode %s",
        ),
        lowerThresholdForBOBF=dict(
            argstr="--lowerThresholdForBOBF %d",
        ),
        makeBOBF=dict(
            argstr="--makeBOBF ",
        ),
        max_step_length=dict(
            argstr="--max_step_length %f",
        ),
        medianFilterSize=dict(
            argstr="--medianFilterSize %s",
            sep=",",
        ),
        minimumFixedPyramid=dict(
            argstr="--minimumFixedPyramid %s",
            sep=",",
        ),
        minimumMovingPyramid=dict(
            argstr="--minimumMovingPyramid %s",
            sep=",",
        ),
        movingBinaryVolume=dict(
            argstr="--movingBinaryVolume %s",
            extensions=None,
        ),
        movingVolume=dict(
            argstr="--movingVolume %s...",
        ),
        neighborhoodForBOBF=dict(
            argstr="--neighborhoodForBOBF %s",
            sep=",",
        ),
        numberOfBCHApproximationTerms=dict(
            argstr="--numberOfBCHApproximationTerms %d",
        ),
        numberOfHistogramBins=dict(
            argstr="--numberOfHistogramBins %d",
        ),
        numberOfMatchPoints=dict(
            argstr="--numberOfMatchPoints %d",
        ),
        numberOfPyramidLevels=dict(
            argstr="--numberOfPyramidLevels %d",
        ),
        numberOfThreads=dict(
            argstr="--numberOfThreads %d",
        ),
        outputCheckerboardVolume=dict(
            argstr="--outputCheckerboardVolume %s",
            hash_files=False,
        ),
        outputDebug=dict(
            argstr="--outputDebug ",
        ),
        outputDisplacementFieldPrefix=dict(
            argstr="--outputDisplacementFieldPrefix %s",
        ),
        outputDisplacementFieldVolume=dict(
            argstr="--outputDisplacementFieldVolume %s",
            hash_files=False,
        ),
        outputNormalized=dict(
            argstr="--outputNormalized ",
        ),
        outputPixelType=dict(
            argstr="--outputPixelType %s",
        ),
        outputVolume=dict(
            argstr="--outputVolume %s",
            hash_files=False,
        ),
        promptUser=dict(
            argstr="--promptUser ",
        ),
        registrationFilterType=dict(
            argstr="--registrationFilterType %s",
        ),
        seedForBOBF=dict(
            argstr="--seedForBOBF %s",
            sep=",",
        ),
        smoothDisplacementFieldSigma=dict(
            argstr="--smoothDisplacementFieldSigma %f",
        ),
        upFieldSmoothing=dict(
            argstr="--upFieldSmoothing %f",
        ),
        upperThresholdForBOBF=dict(
            argstr="--upperThresholdForBOBF %d",
        ),
        use_vanilla_dem=dict(
            argstr="--use_vanilla_dem ",
        ),
        weightFactors=dict(
            argstr="--weightFactors %s",
            sep=",",
        ),
    )
    inputs = VBRAINSDemonWarp.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_VBRAINSDemonWarp_outputs():
    output_map = dict(
        outputCheckerboardVolume=dict(
            extensions=None,
        ),
        outputDisplacementFieldVolume=dict(
            extensions=None,
        ),
        outputVolume=dict(
            extensions=None,
        ),
    )
    outputs = VBRAINSDemonWarp.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..preprocess import TCorrMap


def test_TCorrMap_inputs():
    input_map = dict(
        absolute_threshold=dict(
            argstr="-Thresh %f %s",
            extensions=None,
            name_source="in_file",
            suffix="_thresh",
            xor=(
                "absolute_threshold",
                "var_absolute_threshold",
                "var_absolute_threshold_normalize",
            ),
        ),
        args=dict(
            argstr="%s",
        ),
        automask=dict(
            argstr="-automask",
        ),
        average_expr=dict(
            argstr="-Aexpr %s %s",
            extensions=None,
            name_source="in_file",
            suffix="_aexpr",
            xor=("average_expr", "average_expr_nonzero", "sum_expr"),
        ),
        average_expr_nonzero=dict(
            argstr="-Cexpr %s %s",
            extensions=None,
            name_source="in_file",
            suffix="_cexpr",
            xor=("average_expr", "average_expr_nonzero", "sum_expr"),
        ),
        bandpass=dict(
            argstr="-bpass %f %f",
        ),
        blur_fwhm=dict(
            argstr="-Gblur %f",
        ),
        correlation_maps=dict(
            argstr="-CorrMap %s",
            extensions=None,
            name_source="in_file",
        ),
        correlation_maps_masked=dict(
            argstr="-CorrMask %s",
            extensions=None,
            name_source="in_file",
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        expr=dict(),
        histogram=dict(
            argstr="-Hist %d %s",
            extensions=None,
            name_source="in_file",
            suffix="_hist",
        ),
        histogram_bin_numbers=dict(),
        in_file=dict(
            argstr="-input %s",
            copyfile=False,
            extensions=None,
            mandatory=True,
        ),
        mask=dict(
            argstr="-mask %s",
            extensions=None,
        ),
        mean_file=dict(
            argstr="-Mean %s",
            extensions=None,
            name_source="in_file",
            suffix="_mean",
        ),
        num_threads=dict(
            nohash=True,
            usedefault=True,
        ),
        out_file=dict(
            argstr="-prefix %s",
            extensions=None,
            name_source=["in_file"],
            name_template="%s_afni",
        ),
        outputtype=dict(),
        pmean=dict(
            argstr="-Pmean %s",
            extensions=None,
            name_source="in_file",
            suffix="_pmean",
        ),
        polort=dict(
            argstr="-polort %d",
        ),
        qmean=dict(
            argstr="-Qmean %s",
            extensions=None,
            name_source="in_file",
            suffix="_qmean",
        ),
        regress_out_timeseries=dict(
            argstr="-ort %s",
            extensions=None,
        ),
        seeds=dict(
            argstr="-seed %s",
            extensions=None,
            xor="seeds_width",
        ),
        seeds_width=dict(
            argstr="-Mseed %f",
            xor="seeds",
        ),
        sum_expr=dict(
            argstr="-Sexpr %s %s",
            extensions=None,
            name_source="in_file",
            suffix="_sexpr",
            xor=("average_expr", "average_expr_nonzero", "sum_expr"),
        ),
        thresholds=dict(),
        var_absolute_threshold=dict(
            argstr="-VarThresh %f %f %f %s",
            extensions=None,
            name_source="in_file",
            suffix="_varthresh",
            xor=(
                "absolute_threshold",
                "var_absolute_threshold",
                "var_absolute_threshold_normalize",
            ),
        ),
        var_absolute_threshold_normalize=dict(
            argstr="-VarThreshN %f %f %f %s",
            extensions=None,
            name_source="in_file",
            suffix="_varthreshn",
            xor=(
                "absolute_threshold",
                "var_absolute_threshold",
                "var_absolute_threshold_normalize",
            ),
        ),
        zmean=dict(
            argstr="-Zmean %s",
            extensions=None,
            name_source="in_file",
            suffix="_zmean",
        ),
    )
    inputs = TCorrMap.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_TCorrMap_outputs():
    output_map = dict(
        absolute_threshold=dict(
            extensions=None,
        ),
        average_expr=dict(
            extensions=None,
        ),
        average_expr_nonzero=dict(
            extensions=None,
        ),
        correlation_maps=dict(
            extensions=None,
        ),
        correlation_maps_masked=dict(
            extensions=None,
        ),
        histogram=dict(
            extensions=None,
        ),
        mean_file=dict(
            extensions=None,
        ),
        pmean=dict(
            extensions=None,
        ),
        qmean=dict(
            extensions=None,
        ),
        sum_expr=dict(
            extensions=None,
        ),
        var_absolute_threshold=dict(
            extensions=None,
        ),
        var_absolute_threshold_normalize=dict(
            extensions=None,
        ),
        zmean=dict(
            extensions=None,
        ),
    )
    outputs = TCorrMap.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..gtract import gtractFastMarchingTracking


def test_gtractFastMarchingTracking_inputs():
    input_map = dict(
        args=dict(
            argstr="%s",
        ),
        costStepSize=dict(
            argstr="--costStepSize %f",
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        inputAnisotropyVolume=dict(
            argstr="--inputAnisotropyVolume %s",
            extensions=None,
        ),
        inputCostVolume=dict(
            argstr="--inputCostVolume %s",
            extensions=None,
        ),
        inputStartingSeedsLabelMapVolume=dict(
            argstr="--inputStartingSeedsLabelMapVolume %s",
            extensions=None,
        ),
        inputTensorVolume=dict(
            argstr="--inputTensorVolume %s",
            extensions=None,
        ),
        maximumStepSize=dict(
            argstr="--maximumStepSize %f",
        ),
        minimumStepSize=dict(
            argstr="--minimumStepSize %f",
        ),
        numberOfIterations=dict(
            argstr="--numberOfIterations %d",
        ),
        numberOfThreads=dict(
            argstr="--numberOfThreads %d",
        ),
        outputTract=dict(
            argstr="--outputTract %s",
            hash_files=False,
        ),
        seedThreshold=dict(
            argstr="--seedThreshold %f",
        ),
        startingSeedsLabel=dict(
            argstr="--startingSeedsLabel %d",
        ),
        trackingThreshold=dict(
            argstr="--trackingThreshold %f",
        ),
        writeXMLPolyDataFile=dict(
            argstr="--writeXMLPolyDataFile ",
        ),
    )
    inputs = gtractFastMarchingTracking.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_gtractFastMarchingTracking_outputs():
    output_map = dict(
        outputTract=dict(
            extensions=None,
        ),
    )
    outputs = gtractFastMarchingTracking.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..maxcurvature import maxcurvature


def test_maxcurvature_inputs():
    input_map = dict(
        args=dict(
            argstr="%s",
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        image=dict(
            argstr="--image %s",
            extensions=None,
        ),
        output=dict(
            argstr="--output %s",
            hash_files=False,
        ),
        sigma=dict(
            argstr="--sigma %f",
        ),
        verbose=dict(
            argstr="--verbose ",
        ),
    )
    inputs = maxcurvature.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_maxcurvature_outputs():
    output_map = dict(
        output=dict(
            extensions=None,
        ),
    )
    outputs = maxcurvature.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..dti import ModelFit


def test_ModelFit_inputs():
    input_map = dict(
        args=dict(
            argstr="%s",
        ),
        bgmask=dict(
            argstr="-bgmask %s",
            extensions=None,
        ),
        bgthresh=dict(
            argstr="-bgthresh %G",
        ),
        cfthresh=dict(
            argstr="-csfthresh %G",
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        fixedbvalue=dict(
            argstr="-fixedbvalue %s",
        ),
        fixedmodq=dict(
            argstr="-fixedmod %s",
        ),
        in_file=dict(
            argstr="-inputfile %s",
            extensions=None,
            mandatory=True,
        ),
        inputdatatype=dict(
            argstr="-inputdatatype %s",
        ),
        model=dict(
            argstr="-model %s",
            mandatory=True,
        ),
        noisemap=dict(
            argstr="-noisemap %s",
            extensions=None,
        ),
        out_file=dict(
            argstr="> %s",
            extensions=None,
            genfile=True,
            position=-1,
        ),
        outlier=dict(
            argstr="-outliermap %s",
            extensions=None,
        ),
        outputfile=dict(
            argstr="-outputfile %s",
            extensions=None,
        ),
        residualmap=dict(
            argstr="-residualmap %s",
            extensions=None,
        ),
        scheme_file=dict(
            argstr="-schemefile %s",
            extensions=None,
            mandatory=True,
        ),
        sigma=dict(
            argstr="-sigma %G",
        ),
        tau=dict(
            argstr="-tau %G",
        ),
    )
    inputs = ModelFit.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_ModelFit_outputs():
    output_map = dict(
        fitted_data=dict(
            extensions=None,
        ),
    )
    outputs = ModelFit.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..utils import FWHMx


def test_FWHMx_inputs():
    input_map = dict(
        acf=dict(
            argstr="-acf",
            usedefault=True,
        ),
        args=dict(
            argstr="%s",
        ),
        arith=dict(
            argstr="-arith",
            xor=["geom"],
        ),
        automask=dict(
            argstr="-automask",
            usedefault=True,
        ),
        combine=dict(
            argstr="-combine",
        ),
        compat=dict(
            argstr="-compat",
        ),
        demed=dict(
            argstr="-demed",
            xor=["detrend"],
        ),
        detrend=dict(
            argstr="-detrend",
            usedefault=True,
            xor=["demed"],
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        geom=dict(
            argstr="-geom",
            xor=["arith"],
        ),
        in_file=dict(
            argstr="-input %s",
            extensions=None,
            mandatory=True,
        ),
        mask=dict(
            argstr="-mask %s",
            extensions=None,
        ),
        out_detrend=dict(
            argstr="-detprefix %s",
            extensions=None,
            keep_extension=False,
            name_source="in_file",
            name_template="%s_detrend",
        ),
        out_file=dict(
            argstr="> %s",
            extensions=None,
            keep_extension=False,
            name_source="in_file",
            name_template="%s_fwhmx.out",
            position=-1,
        ),
        out_subbricks=dict(
            argstr="-out %s",
            extensions=None,
            keep_extension=False,
            name_source="in_file",
            name_template="%s_subbricks.out",
        ),
        unif=dict(
            argstr="-unif",
        ),
    )
    inputs = FWHMx.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_FWHMx_outputs():
    output_map = dict(
        acf_param=dict(),
        fwhm=dict(),
        out_acf=dict(
            extensions=None,
        ),
        out_detrend=dict(
            extensions=None,
        ),
        out_file=dict(
            extensions=None,
        ),
        out_subbricks=dict(
            extensions=None,
        ),
    )
    outputs = FWHMx.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..minc import ToEcat


def test_ToEcat_inputs():
    input_map = dict(
        args=dict(
            argstr="%s",
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        ignore_acquisition_variable=dict(
            argstr="-ignore_acquisition_variable",
        ),
        ignore_ecat_acquisition_variable=dict(
            argstr="-ignore_ecat_acquisition_variable",
        ),
        ignore_ecat_main=dict(
            argstr="-ignore_ecat_main",
        ),
        ignore_ecat_subheader_variable=dict(
            argstr="-ignore_ecat_subheader_variable",
        ),
        ignore_patient_variable=dict(
            argstr="-ignore_patient_variable",
        ),
        ignore_study_variable=dict(
            argstr="-ignore_study_variable",
        ),
        input_file=dict(
            argstr="%s",
            extensions=None,
            mandatory=True,
            position=-2,
        ),
        no_decay_corr_fctr=dict(
            argstr="-no_decay_corr_fctr",
        ),
        output_file=dict(
            argstr="%s",
            extensions=None,
            genfile=True,
            hash_files=False,
            keep_extension=False,
            name_source=["input_file"],
            name_template="%s_to_ecat.v",
            position=-1,
        ),
        voxels_as_integers=dict(
            argstr="-label",
        ),
    )
    inputs = ToEcat.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_ToEcat_outputs():
    output_map = dict(
        output_file=dict(
            extensions=None,
        ),
    )
    outputs = ToEcat.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..utils import AI


def test_AI_inputs():
    input_map = dict(
        args=dict(
            argstr="%s",
        ),
        convergence=dict(
            argstr="-c [%d,%g,%d]",
            usedefault=True,
        ),
        dimension=dict(
            argstr="-d %d",
            usedefault=True,
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        fixed_image=dict(
            extensions=None,
            mandatory=True,
        ),
        fixed_image_mask=dict(
            argstr="-x %s",
            extensions=None,
        ),
        metric=dict(
            argstr="-m %s",
            mandatory=True,
        ),
        moving_image=dict(
            extensions=None,
            mandatory=True,
        ),
        moving_image_mask=dict(
            extensions=None,
            requires=["fixed_image_mask"],
        ),
        num_threads=dict(
            nohash=True,
            usedefault=True,
        ),
        output_transform=dict(
            argstr="-o %s",
            extensions=None,
            usedefault=True,
        ),
        principal_axes=dict(
            argstr="-p %d",
            usedefault=True,
            xor=["blobs"],
        ),
        search_factor=dict(
            argstr="-s [%g,%g]",
            usedefault=True,
        ),
        search_grid=dict(
            argstr="-g %s",
            min_ver="2.3.0",
        ),
        transform=dict(
            argstr="-t %s[%g]",
            usedefault=True,
        ),
        verbose=dict(
            argstr="-v %d",
            usedefault=True,
        ),
    )
    inputs = AI.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_AI_outputs():
    output_map = dict(
        output_transform=dict(
            extensions=None,
        ),
    )
    outputs = AI.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
# -*- coding: utf8 -*-
"""Autogenerated file - DO NOT EDIT
If you spot a bug, please report it on the mailing list and/or change the generator."""

from nipype.interfaces.base import (
    CommandLine,
    CommandLineInputSpec,
    SEMLikeCommandLine,
    TraitedSpec,
    File,
    Directory,
    traits,
    isdefined,
    InputMultiPath,
    OutputMultiPath,
)
import os


class BSplineToDeformationFieldInputSpec(CommandLineInputSpec):
    tfm = File(exists=True, argstr="--tfm %s")
    refImage = File(exists=True, argstr="--refImage %s")
    defImage = traits.Either(
        traits.Bool, File(), hash_files=False, argstr="--defImage %s"
    )


class BSplineToDeformationFieldOutputSpec(TraitedSpec):
    defImage = File(exists=True)


class BSplineToDeformationField(SEMLikeCommandLine):
    """title: BSpline to deformation field

    category: Legacy.Converters

    description: Create a dense deformation field from a bspline+bulk transform.

    version: 0.1.0.$Revision: 2104 $(alpha)

    documentation-url: http://wiki.slicer.org/slicerWiki/index.php/Documentation/4.1/Modules/BSplineToDeformationField

    contributor: Andrey Fedorov (SPL, BWH)

    acknowledgements: This work is funded by NIH grants R01 CA111288 and U01 CA151261.
    """

    input_spec = BSplineToDeformationFieldInputSpec
    output_spec = BSplineToDeformationFieldOutputSpec
    _cmd = "BSplineToDeformationField "
    _outputs_filenames = {"defImage": "defImage.nii"}



# -*- coding: utf-8 -*-
# Copyright 2008 Lowell Alleman
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may not
#   use this file except in compliance with the License. You may obtain a copy
#   of the License at http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
""" cloghandler.py:  A smart replacement for the standard RotatingFileHandler

ConcurrentRotatingFileHandler:  This class is a log handler which is a drop-in
replacement for the python standard log handler 'RotateFileHandler', the primary
difference being that this handler will continue to write to the same file if
the file cannot be rotated for some reason, whereas the RotatingFileHandler will
strictly adhere to the maximum file size.  Unfortunately, if you are using the
RotatingFileHandler on Windows, you will find that once an attempted rotation
fails, all subsequent log messages are dropped.  The other major advantage of
this module is that multiple processes can safely write to a single log file.

To put it another way:  This module's top priority is preserving your log
records, whereas the standard library attempts to limit disk usage, which can
potentially drop log messages. If you are trying to determine which module to
use, there are number of considerations: What is most important: strict disk
space usage or preservation of log messages? What OSes are you supporting? Can
you afford to have processes blocked by file locks?

Concurrent access is handled by using file locks, which should ensure that log
messages are not dropped or clobbered. This means that a file lock is acquired
and released for every log message that is written to disk. (On Windows, you may
also run into a temporary situation where the log file must be opened and closed
for each log message.) This can have potentially performance implications. In my
testing, performance was more than adequate, but if you need a high-volume or
low-latency solution, I suggest you look elsewhere.

See the README file for an example usage of this module.

"""

from builtins import range

__version__ = "$Id: cloghandler.py 6175 2009-11-02 18:40:35Z lowell $"
__author__ = "Lowell Alleman"
__all__ = ["ConcurrentRotatingFileHandler"]

import os
import sys
from random import randint
from logging import Handler
from logging.handlers import BaseRotatingHandler

try:
    import codecs
except ImportError:
    codecs = None

from filelock import SoftFileLock

# A client can set this to true to automatically convert relative paths to
# absolute paths (which will also hide the absolute path warnings)
FORCE_ABSOLUTE_PATH = False


class ConcurrentRotatingFileHandler(BaseRotatingHandler):
    """
    Handler for logging to a set of files, which switches from one file to the
    next when the current file reaches a certain size. Multiple processes can
    write to the log file concurrently, but this may mean that the file will
    exceed the given size.
    """

    def __init__(
        self,
        filename,
        mode="a",
        maxBytes=0,
        backupCount=0,
        encoding=None,
        debug=True,
        supress_abs_warn=False,
    ):
        """
        Open the specified file and use it as the stream for logging.

        By default, the file grows indefinitely. You can specify particular
        values of maxBytes and backupCount to allow the file to rollover at
        a predetermined size.

        Rollover occurs whenever the current log file is nearly maxBytes in
        length. If backupCount is >= 1, the system will successively create
        new files with the same pathname as the base file, but with extensions
        ".1", ".2" etc. appended to it. For example, with a backupCount of 5
        and a base file name of "app.log", you would get "app.log",
        "app.log.1", "app.log.2", ... through to "app.log.5". The file being
        written to is always "app.log" - when it gets filled up, it is closed
        and renamed to "app.log.1", and if files "app.log.1", "app.log.2" etc.
        exist, then they are renamed to "app.log.2", "app.log.3" etc.
        respectively.

        If maxBytes is zero, rollover never occurs.

        On Windows, it is not possible to rename a file that is currently opened
        by another process.  This means that it is not possible to rotate the
        log files if multiple processes is using the same log file.  In this
        case, the current log file will continue to grow until the rotation can
        be completed successfully.  In order for rotation to be possible, all of
        the other processes need to close the file first.  A mechanism, called
        "degraded" mode, has been created for this scenario.  In degraded mode,
        the log file is closed after each log message is written.  So once all
        processes have entered degraded mode, the next rotate log attempt should
        be successful and then normal logging can be resumed.

        This log handler assumes that all concurrent processes logging to a
        single file will are using only this class, and that the exact same
        parameters are provided to each instance of this class.  If, for
        example, two different processes are using this class, but with
        different values for 'maxBytes' or 'backupCount', then odd behavior is
        expected. The same is true if this class is used by one application, but
        the RotatingFileHandler is used by another.

        NOTE:  You should always provide 'filename' as an absolute path, since
        this class will need to re-open the file during rotation. If your
        application call os.chdir() then subsequent log files could be created
        in the wrong directory.
        """
        # The question of absolute paths: I'm not sure what the 'right thing' is
        # to do here. RotatingFileHander simply ignores this possibility. I was
        # going call os.path.abspath(), but that potentially limits uses. For
        # example, on Linux (any posix system?) you can rename a directory of a
        # running app, and the app wouldn't notice as long as it only opens new
        # files using relative paths. But since that's not a "normal" thing to
        # do, and having an app call os.chdir() is a much more likely scenario
        # that should be supported. For the moment, we are just going to warn
        # the user if they provide a relative path and do some other voodoo
        # logic that you'll just have to review for yourself.

        # if the given filename contains no path, we make an absolute path
        if not os.path.isabs(filename):
            if FORCE_ABSOLUTE_PATH or not os.path.split(filename)[0]:
                filename = os.path.abspath(filename)
            elif not supress_abs_warn:
                from warnings import warn

                warn(
                    "The given 'filename' should be an absolute path.  If your "
                    "application calls os.chdir(), your logs may get messed up. "
                    "Use 'supress_abs_warn=True' to hide this message."
                )
        try:
            BaseRotatingHandler.__init__(self, filename, mode, encoding)
        except TypeError:  # Due to a different logging release without encoding support  (Python 2.4.1 and earlier?)
            BaseRotatingHandler.__init__(self, filename, mode)
            self.encoding = encoding

        self._rotateFailed = False
        self.maxBytes = maxBytes
        self.backupCount = backupCount
        # Prevent multiple extensions on the lock file (Only handles the normal "*.log" case.)
        self.lock_file = "%s.lock" % filename
        self.stream_lock = SoftFileLock(self.lock_file)

        # For debug mode, swap out the "_degrade()" method with a more a verbose one.
        if debug:
            self._degrade = self._degrade_debug

    def _openFile(self, mode):
        if self.encoding:
            self.stream = codecs.open(self.baseFilename, mode, self.encoding)
        else:
            self.stream = open(self.baseFilename, mode)

    def acquire(self):
        """Acquire thread and file locks. Also re-opening log file when running
        in 'degraded' mode."""
        # handle thread lock
        Handler.acquire(self)
        self.stream_lock.acquire()
        if self.stream.closed:
            self._openFile(self.mode)

    def release(self):
        """Release file and thread locks. Flush stream and take care of closing
        stream in 'degraded' mode."""
        try:
            if not self.stream.closed:
                self.stream.flush()
                if self._rotateFailed:
                    self.stream.close()
        except IOError:
            if self._rotateFailed:
                self.stream.close()
        finally:
            try:
                self.stream_lock.release()
            finally:
                # release thread lock
                Handler.release(self)

    def close(self):
        """
        Closes the stream.
        """
        if not self.stream.closed:
            self.stream.flush()
            self.stream.close()
        Handler.close(self)

    def flush(self):
        """flush():  Do nothing.

        Since a flush is issued in release(), we don't do it here. To do a flush
        here, it would be necessary to re-lock everything, and it is just easier
        and cleaner to do it all in release(), rather than requiring two lock
        ops per handle() call.

        Doing a flush() here would also introduces a window of opportunity for
        another process to write to the log file in between calling
        stream.write() and stream.flush(), which seems like a bad thing."""
        pass

    def _degrade(self, degrade, msg, *args):
        """Set degrade mode or not.  Ignore msg."""
        self._rotateFailed = degrade
        del msg, args  # avoid pychecker warnings

    def _degrade_debug(self, degrade, msg, *args):
        """A more colorful version of _degade(). (This is enabled by passing
        "debug=True" at initialization).
        """
        if degrade:
            if not self._rotateFailed:
                sys.stderr.write(
                    "Degrade mode - ENTERING - (pid=%d)  %s\n"
                    % (os.getpid(), msg % args)
                )
                self._rotateFailed = True
        else:
            if self._rotateFailed:
                sys.stderr.write(
                    "Degrade mode - EXITING  - (pid=%d)   %s\n"
                    % (os.getpid(), msg % args)
                )
                self._rotateFailed = False

    def doRollover(self):
        """
        Do a rollover, as described in __init__().
        """
        if self.backupCount <= 0:
            # Don't keep any backups, just overwrite the existing backup file
            # Locking doesn't much matter here; since we are overwriting it anyway
            self.stream.close()
            self._openFile("w")
            return
        self.stream.close()
        try:
            # Attempt to rename logfile to tempname:  There is a slight race-condition here, but it seems unavoidable
            tmpname = None
            while not tmpname or os.path.exists(tmpname):
                tmpname = "%s.rotate.%08d" % (self.baseFilename, randint(0, 99999999))
            try:
                # Do a rename test to determine if we can successfully rename the log file
                os.rename(self.baseFilename, tmpname)
            except (IOError, OSError):
                exc_value = sys.exc_info()[1]
                self._degrade(
                    True, "rename failed.  File in use?  " "exception=%s", exc_value
                )
                return

            # Q: Is there some way to protect this code from a KeboardInterupt?
            # This isn't necessarily a data loss issue, but it certainly would
            # break the rotation process during my stress testing.

            # There is currently no mechanism in place to handle the situation
            # where one of these log files cannot be renamed. (Example, user
            # opens "logfile.3" in notepad)
            for i in range(self.backupCount - 1, 0, -1):
                sfn = "%s.%d" % (self.baseFilename, i)
                dfn = "%s.%d" % (self.baseFilename, i + 1)
                if os.path.exists(sfn):
                    # print "%s -> %s" % (sfn, dfn)
                    if os.path.exists(dfn):
                        os.remove(dfn)
                    os.rename(sfn, dfn)
            dfn = self.baseFilename + ".1"
            if os.path.exists(dfn):
                os.remove(dfn)
            os.rename(tmpname, dfn)
            # print "%s -> %s" % (self.baseFilename, dfn)
            self._degrade(False, "Rotation completed")
        finally:
            self._openFile(self.mode)

    def shouldRollover(self, record):
        """
        Determine if rollover should occur.

        For those that are keeping track. This differs from the standard
        library's RotatingLogHandler class. Because there is no promise to keep
        the file size under maxBytes we ignore the length of the current record.
        """
        del record  # avoid pychecker warnings
        if self._shouldRollover():
            # if some other process already did the rollover we might
            # checked log.1, so we reopen the stream and check again on
            # the right log file
            self.stream.close()
            self._openFile(self.mode)
            return self._shouldRollover()
        return False

    def _shouldRollover(self):
        if self.maxBytes > 0:  # are we rolling over?
            try:
                self.stream.seek(0, 2)  # due to non-posix-compliant Windows feature
            except IOError:
                return True
            if self.stream.tell() >= self.maxBytes:
                return True
            else:
                self._degrade(False, "Rotation done or not needed at this time")
        return False


# Publish this class to the "logging.handlers" module so that it can be use
# from a logging config file via logging.config.fileConfig().
import logging.handlers

logging.handlers.ConcurrentRotatingFileHandler = ConcurrentRotatingFileHandler



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..utils import Cat


def test_Cat_inputs():
    input_map = dict(
        args=dict(
            argstr="%s",
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        in_files=dict(
            argstr="%s",
            mandatory=True,
            position=-2,
        ),
        keepfree=dict(
            argstr="-nonfixed",
        ),
        num_threads=dict(
            nohash=True,
            usedefault=True,
        ),
        omitconst=dict(
            argstr="-nonconst",
        ),
        out_cint=dict(
            xor=["out_format", "out_nice", "out_double", "out_fint", "out_int"],
        ),
        out_double=dict(
            argstr="-d",
            xor=["out_format", "out_nice", "out_int", "out_fint", "out_cint"],
        ),
        out_file=dict(
            argstr="> %s",
            extensions=None,
            mandatory=True,
            position=-1,
            usedefault=True,
        ),
        out_fint=dict(
            argstr="-f",
            xor=["out_format", "out_nice", "out_double", "out_int", "out_cint"],
        ),
        out_format=dict(
            argstr="-form %s",
            xor=["out_int", "out_nice", "out_double", "out_fint", "out_cint"],
        ),
        out_int=dict(
            argstr="-i",
            xor=["out_format", "out_nice", "out_double", "out_fint", "out_cint"],
        ),
        out_nice=dict(
            argstr="-n",
            xor=["out_format", "out_int", "out_double", "out_fint", "out_cint"],
        ),
        outputtype=dict(),
        sel=dict(
            argstr="-sel %s",
        ),
        stack=dict(
            argstr="-stack",
        ),
    )
    inputs = Cat.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_Cat_outputs():
    output_map = dict(
        out_file=dict(
            extensions=None,
        ),
    )
    outputs = Cat.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
""" class mixin and utilities for enabling reports for nipype interfaces """

import os
from abc import abstractmethod

from ... import logging
from ..base import File, BaseInterface, BaseInterfaceInputSpec, TraitedSpec

iflogger = logging.getLogger("nipype.interface")


class ReportCapableInputSpec(BaseInterfaceInputSpec):
    out_report = File(
        "report",
        usedefault=True,
        hash_files=False,
        desc="filename for the visual report",
    )


class ReportCapableOutputSpec(TraitedSpec):
    out_report = File(desc="filename for the visual report")


class ReportCapableInterface(BaseInterface):
    """Mixin to enable reporting for Nipype interfaces"""

    _out_report = None

    def __init__(self, generate_report=False, **kwargs):
        super(ReportCapableInterface, self).__init__(**kwargs)
        self.generate_report = generate_report

    def _post_run_hook(self, runtime):
        runtime = super(ReportCapableInterface, self)._post_run_hook(runtime)

        # leave early if there's nothing to do
        if not self.generate_report:
            return runtime

        self._out_report = self.inputs.out_report
        if not os.path.isabs(self._out_report):
            self._out_report = os.path.abspath(
                os.path.join(runtime.cwd, self._out_report)
            )

        self._generate_report()

        return runtime

    def _list_outputs(self):
        try:
            outputs = super(ReportCapableInterface, self)._list_outputs()
        except NotImplementedError:
            outputs = {}
        if self._out_report is not None:
            outputs["out_report"] = self._out_report
        return outputs

    @abstractmethod
    def _generate_report(self):
        """
        Saves report to file identified by _out_report instance variable
        """
        raise NotImplementedError



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
import os

import pytest
from nipype.testing.fixtures import create_files_in_directory

from nipype.interfaces import freesurfer
from nipype.interfaces.freesurfer import Info
from nipype import LooseVersion


@pytest.mark.skipif(freesurfer.no_freesurfer(), reason="freesurfer is not installed")
def test_robustregister(create_files_in_directory):
    filelist, outdir = create_files_in_directory

    reg = freesurfer.RobustRegister()
    cwd = os.getcwd()

    # make sure command gets called
    assert reg.cmd == "mri_robust_register"

    # test raising error with mandatory args absent
    with pytest.raises(ValueError):
        reg.run()

    # .inputs based parameters setting
    reg.inputs.source_file = filelist[0]
    reg.inputs.target_file = filelist[1]
    reg.inputs.auto_sens = True
    assert reg.cmdline == (
        "mri_robust_register --satit --lta "
        "%s/%s_robustreg.lta --mov %s --dst %s"
        % (cwd, filelist[0][:-4], filelist[0], filelist[1])
    )

    # constructor based parameter setting
    reg2 = freesurfer.RobustRegister(
        source_file=filelist[0],
        target_file=filelist[1],
        outlier_sens=3.0,
        out_reg_file="foo.lta",
        half_targ=True,
    )
    assert reg2.cmdline == (
        "mri_robust_register --halfdst %s_halfway.nii --lta foo.lta "
        "--sat 3.0000 --mov %s --dst %s"
        % (os.path.join(outdir, filelist[1][:-4]), filelist[0], filelist[1])
    )


@pytest.mark.skipif(freesurfer.no_freesurfer(), reason="freesurfer is not installed")
def test_fitmsparams(create_files_in_directory):
    filelist, outdir = create_files_in_directory

    fit = freesurfer.FitMSParams()

    # make sure command gets called
    assert fit.cmd == "mri_ms_fitparms"

    # test raising error with mandatory args absent
    with pytest.raises(ValueError):
        fit.run()

    # .inputs based parameters setting
    fit.inputs.in_files = filelist
    fit.inputs.out_dir = outdir
    assert fit.cmdline == "mri_ms_fitparms  %s %s %s" % (
        filelist[0],
        filelist[1],
        outdir,
    )

    # constructor based parameter setting
    fit2 = freesurfer.FitMSParams(
        in_files=filelist, te_list=[1.5, 3.5], flip_list=[20, 30], out_dir=outdir
    )
    assert fit2.cmdline == (
        "mri_ms_fitparms  -te %.3f -fa %.1f %s -te %.3f -fa %.1f %s %s"
        % (1.500, 20.0, filelist[0], 3.500, 30.0, filelist[1], outdir)
    )


@pytest.mark.skipif(freesurfer.no_freesurfer(), reason="freesurfer is not installed")
def test_synthesizeflash(create_files_in_directory):
    filelist, outdir = create_files_in_directory

    syn = freesurfer.SynthesizeFLASH()

    # make sure command gets called
    assert syn.cmd == "mri_synthesize"

    # test raising error with mandatory args absent
    with pytest.raises(ValueError):
        syn.run()

    # .inputs based parameters setting
    syn.inputs.t1_image = filelist[0]
    syn.inputs.pd_image = filelist[1]
    syn.inputs.flip_angle = 30
    syn.inputs.te = 4.5
    syn.inputs.tr = 20

    assert syn.cmdline == (
        "mri_synthesize 20.00 30.00 4.500 %s %s %s"
        % (filelist[0], filelist[1], os.path.join(outdir, "synth-flash_30.mgz"))
    )

    # constructor based parameters setting
    syn2 = freesurfer.SynthesizeFLASH(
        t1_image=filelist[0], pd_image=filelist[1], flip_angle=20, te=5, tr=25
    )
    assert syn2.cmdline == (
        "mri_synthesize 25.00 20.00 5.000 %s %s %s"
        % (filelist[0], filelist[1], os.path.join(outdir, "synth-flash_20.mgz"))
    )


@pytest.mark.skipif(freesurfer.no_freesurfer(), reason="freesurfer is not installed")
def test_mandatory_outvol(create_files_in_directory):
    filelist, outdir = create_files_in_directory
    mni = freesurfer.MNIBiasCorrection()

    # make sure command gets called
    assert mni.cmd == "mri_nu_correct.mni"

    # test raising error with mandatory args absent
    with pytest.raises(ValueError):
        mni.cmdline

    # test with minimal args
    mni.inputs.in_file = filelist[0]
    base, ext = os.path.splitext(os.path.basename(filelist[0]))
    if ext == ".gz":
        base, ext2 = os.path.splitext(base)
        ext = ext2 + ext

    assert mni.cmdline == (
        "mri_nu_correct.mni --i %s --n 4 --o %s_output%s" % (filelist[0], base, ext)
    )

    # test with custom outfile
    mni.inputs.out_file = "new_corrected_file.mgz"
    assert mni.cmdline == (
        "mri_nu_correct.mni --i %s --n 4 --o new_corrected_file.mgz" % (filelist[0])
    )

    # constructor based tests
    mni2 = freesurfer.MNIBiasCorrection(
        in_file=filelist[0], out_file="bias_corrected_output", iterations=2
    )
    assert mni2.cmdline == (
        "mri_nu_correct.mni --i %s --n 2 --o bias_corrected_output" % filelist[0]
    )


@pytest.mark.skipif(freesurfer.no_freesurfer(), reason="freesurfer is not installed")
def test_bbregister(create_files_in_directory):
    filelist, outdir = create_files_in_directory
    bbr = freesurfer.BBRegister()

    # make sure command gets called
    assert bbr.cmd == "bbregister"

    # test raising error with mandatory args absent
    with pytest.raises(ValueError):
        bbr.cmdline

    bbr.inputs.subject_id = "fsaverage"
    bbr.inputs.source_file = filelist[0]
    bbr.inputs.contrast_type = "t2"

    # Check that 'init' is mandatory in FS < 6, but not in 6+
    if Info.looseversion() < LooseVersion("6.0.0"):
        with pytest.raises(ValueError):
            bbr.cmdline
    else:
        bbr.cmdline

    bbr.inputs.init = "fsl"

    base, ext = os.path.splitext(os.path.basename(filelist[0]))
    if ext == ".gz":
        base, _ = os.path.splitext(base)

    assert bbr.cmdline == (
        "bbregister --t2 --init-fsl "
        "--reg {base}_bbreg_fsaverage.dat "
        "--mov {full} --s fsaverage".format(full=filelist[0], base=base)
    )


def test_FSVersion():
    """Check that FSVersion is a string that can be compared with LooseVersion"""
    assert isinstance(freesurfer.preprocess.FSVersion, str)
    assert LooseVersion(freesurfer.preprocess.FSVersion) >= LooseVersion("0")



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""Miscellaneous utility functions
"""
import os
import sys
import re
from collections.abc import Iterator
from warnings import warn

from distutils.version import LooseVersion

import numpy as np

try:
    from textwrap import indent as textwrap_indent
except ImportError:

    def textwrap_indent(text, prefix):
        """A textwrap.indent replacement for Python < 3.3"""
        if not prefix:
            return text
        splittext = text.splitlines(True)
        return prefix + prefix.join(splittext)


def human_order_sorted(l):
    """Sorts string in human order (i.e. 'stat10' will go after 'stat2')"""

    def atoi(text):
        return int(text) if text.isdigit() else text

    def natural_keys(text):
        if isinstance(text, tuple):
            text = text[0]
        return [atoi(c) for c in re.split(r"(\d+)", text)]

    return sorted(l, key=natural_keys)


def trim(docstring, marker=None):
    if isinstance(docstring, bytes):
        docstring = str(docstring, "utf-8")

    if not docstring:
        return ""
    # Convert tabs to spaces (following the normal Python rules)
    # and split into a list of lines:
    lines = docstring.expandtabs().splitlines()
    # Determine minimum indentation (first line doesn't count):
    indent = sys.maxsize
    for line in lines[1:]:
        stripped = line.lstrip()
        if stripped:
            indent = min(indent, len(line) - len(stripped))
    # Remove indentation (first line is special):
    trimmed = [lines[0].strip()]
    if indent < sys.maxsize:
        for line in lines[1:]:
            # replace existing REST marker with doc level marker
            stripped = line.lstrip().strip().rstrip()
            if (
                marker is not None
                and stripped
                and all([s == stripped[0] for s in stripped])
                and stripped[0] not in [":"]
            ):
                line = line.replace(stripped[0], marker)
            trimmed.append(line[indent:].rstrip())
    # Strip off trailing and leading blank lines:
    while trimmed and not trimmed[-1]:
        trimmed.pop()
    while trimmed and not trimmed[0]:
        trimmed.pop(0)
    # Return a single string:
    return "\n".join(trimmed)


def find_indices(condition):
    "Return the indices where ravel(condition) is true"
    (res,) = np.nonzero(np.ravel(condition))
    return res


def is_container(item):
    """Checks if item is a container (list, tuple, dict, set)

    Parameters
    ----------
    item : object
        object to check for .__iter__

    Returns
    -------
    output : Boolean
        True if container
        False if not (eg string)
    """
    if isinstance(item, str):
        return False
    elif hasattr(item, "__iter__"):
        return True
    else:
        return False


def container_to_string(cont):
    """Convert a container to a command line string.

    Elements of the container are joined with a space between them,
    suitable for a command line parameter.

    If the container `cont` is only a sequence, like a string and not a
    container, it is returned unmodified.

    Parameters
    ----------
    cont : container
       A container object like a list, tuple, dict, or a set.

    Returns
    -------
    cont_str : string
        Container elements joined into a string.

    """
    if hasattr(cont, "__iter__") and not isinstance(cont, str):
        cont = " ".join(cont)
    return str(cont)


# Dependency checks.  Copied this from Nipy, with some modificiations
# (added app as a parameter).
def package_check(
    pkg_name,
    version=None,
    app=None,
    checker=LooseVersion,
    exc_failed_import=ImportError,
    exc_failed_check=RuntimeError,
):
    """Check that the minimal version of the required package is installed.

    Parameters
    ----------
    pkg_name : string
        Name of the required package.
    version : string, optional
        Minimal version number for required package.
    app : string, optional
        Application that is performing the check.  For instance, the
        name of the tutorial being executed that depends on specific
        packages.  Default is *Nipype*.
    checker : object, optional
        The class that will perform the version checking.  Default is
        distutils.version.LooseVersion.
    exc_failed_import : Exception, optional
        Class of the exception to be thrown if import failed.
    exc_failed_check : Exception, optional
        Class of the exception to be thrown if version check failed.

    Examples
    --------
    package_check('numpy', '1.3')
    package_check('scipy', '0.7', 'tutorial1')

    """

    if app:
        msg = "%s requires %s" % (app, pkg_name)
    else:
        msg = "Nipype requires %s" % pkg_name
    if version:
        msg += " with version >= %s" % (version,)
    try:
        mod = __import__(pkg_name)
    except ImportError as e:
        raise exc_failed_import(msg) from e
    if not version:
        return
    try:
        have_version = mod.__version__
    except AttributeError as e:
        raise exc_failed_check("Cannot find version for %s" % pkg_name) from e
    if checker(have_version) < checker(version):
        raise exc_failed_check(msg)


def str2bool(v):
    """
    Convert strings (and bytearrays) to boolean values

    >>> all([str2bool(v) for v in (True, "yes", "true",
    ...      "y", "t", "Yes", "True", "1", "on", "On")])
    True
    >>> all([str2bool(v.encode('utf-8'))
    ...      for v in ("yes", "true", "y", "t", "1", "Yes", "on", "On")])
    True
    >>> any([str2bool(v) for v in (False, "no", "false", "n", "f",
    ...      "False", "0", "off", "Off")])
    False
    >>> any([str2bool(v.encode('utf-8'))
    ...      for v in ("no", "false", "n", "f", "0", "off", "Off")])
    False
    >>> str2bool(None)  # doctest: +ELLIPSIS
    Traceback (most recent call last):
        ...
    ValueError: ...
    >>> str2bool('/some/path')  # doctest: +ELLIPSIS
    Traceback (most recent call last):
        ...
    ValueError: ...
    >>> str2bool('Agg')  # doctest: +ELLIPSIS
    Traceback (most recent call last):
        ...
    ValueError: ...
    >>> str2bool('INFO')  # doctest: +ELLIPSIS
    Traceback (most recent call last):
        ...
    ValueError: ...
    >>> str2bool('/some/bytes/path'.encode('utf-8'))  # doctest: +ELLIPSIS
    Traceback (most recent call last):
        ...
    ValueError: ...

    """
    if isinstance(v, bool):
        return v

    if isinstance(v, bytes):
        v = v.decode("utf-8")

    if isinstance(v, str):
        lower = v.lower()
        if lower in ("yes", "true", "y", "t", "1", "on"):
            return True
        elif lower in ("no", "false", "n", "f", "0", "off"):
            return False

    raise ValueError("%r cannot be converted to bool" % v)


def flatten(S):
    if S == []:
        return S
    if isinstance(S[0], list):
        return flatten(S[0]) + flatten(S[1:])
    return S[:1] + flatten(S[1:])


def unflatten(in_list, prev_structure):
    if not isinstance(in_list, Iterator):
        in_list = iter(in_list)

    if not isinstance(prev_structure, list):
        return next(in_list)

    out = []
    for item in prev_structure:
        out.append(unflatten(in_list, item))
    return out


def normalize_mc_params(params, source):
    """
    Normalize a single row of motion parameters to the SPM format.

    SPM saves motion parameters as:
        x   Right-Left          (mm)
        y   Anterior-Posterior  (mm)
        z   Superior-Inferior   (mm)
        rx  Pitch               (rad)
        ry  Roll                (rad)
        rz  Yaw                 (rad)
    """
    if source.upper() == "FSL":
        params = params[[3, 4, 5, 0, 1, 2]]
    elif source.upper() in ("AFNI", "FSFAST"):
        params = params[np.asarray([4, 5, 3, 1, 2, 0]) + (len(params) > 6)]
        params[3:] = params[3:] * np.pi / 180.0
    elif source.upper() == "NIPY":
        from nipy.algorithms.registration import to_matrix44, aff2euler

        matrix = to_matrix44(params)
        params = np.zeros(6)
        params[:3] = matrix[:3, 3]
        params[-1:2:-1] = aff2euler(matrix)

    return params


def dict_diff(dold, dnew, indent=0):
    """Helper to log what actually changed from old to new values of
    dictionaries.

    typical use -- log difference for hashed_inputs
    """
    # First check inputs, since they usually are lists of tuples
    # and dicts are required.
    if isinstance(dnew, list):
        dnew = dict(dnew)
    if isinstance(dold, list):
        dold = dict(dold)

    # Compare against hashed_inputs
    # Keys: should rarely differ
    new_keys = set(dnew.keys())
    old_keys = set(dold.keys())

    diff = []
    if new_keys - old_keys:
        diff += ["  * keys not previously seen: %s" % (new_keys - old_keys)]

    if old_keys - new_keys:
        diff += ["  * keys not presently seen: %s" % (old_keys - new_keys)]

    # Add topical message
    if diff:
        diff.insert(0, "Dictionaries had differing keys:")

    diffkeys = len(diff)

    # Values in common keys would differ quite often,
    # so we need to join the messages together
    for k in new_keys.intersection(old_keys):
        try:
            new, old = dnew[k], dold[k]
            same = new == old
            if not same:
                # Since JSON does not discriminate between lists and
                # tuples, we might need to cast them into the same type
                # as the last resort.  And lets try to be more generic
                same = old.__class__(new) == old
        except Exception:
            same = False
        if not same:
            diff += ["  * %s: %r != %r" % (k, dnew[k], dold[k])]

    if len(diff) > diffkeys:
        diff.insert(diffkeys, "Some dictionary entries had differing values:")

    return textwrap_indent("\n".join(diff), " " * indent)


def rgetcwd(error=True):
    """
    Robust replacement for getcwd when folders get removed
    If error==True, this is just an alias for os.getcwd()
    """
    if error:
        return os.getcwd()

    try:
        cwd = os.getcwd()
    except OSError as exc:
        # Changing back to cwd is probably not necessary
        # but this makes sure there's somewhere to change to.
        cwd = os.getenv("PWD")
        if cwd is None:
            raise OSError(
                (
                    exc.errno,
                    "Current directory does not exist anymore, "
                    "and nipype was not able to guess it from the environment",
                )
            )
        warn('Current folder does not exist, replacing with "%s" instead.' % cwd)
    return cwd



# -*- coding: utf-8 -*-
from .specialized import VBRAINSDemonWarp, BRAINSDemonWarp, BRAINSTransformFromFiducials
from .brainsresample import BRAINSResample
from .brainsfit import BRAINSFit
from .brainsresize import BRAINSResize



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..utils import ConvertWarp


def test_ConvertWarp_inputs():
    input_map = dict(
        abswarp=dict(
            argstr="--abs",
            xor=["relwarp"],
        ),
        args=dict(
            argstr="%s",
        ),
        cons_jacobian=dict(
            argstr="--constrainj",
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        jacobian_max=dict(
            argstr="--jmax=%f",
        ),
        jacobian_min=dict(
            argstr="--jmin=%f",
        ),
        midmat=dict(
            argstr="--midmat=%s",
            extensions=None,
        ),
        out_abswarp=dict(
            argstr="--absout",
            xor=["out_relwarp"],
        ),
        out_file=dict(
            argstr="--out=%s",
            extensions=None,
            name_source=["reference"],
            name_template="%s_concatwarp",
            output_name="out_file",
            position=-1,
        ),
        out_relwarp=dict(
            argstr="--relout",
            xor=["out_abswarp"],
        ),
        output_type=dict(),
        postmat=dict(
            argstr="--postmat=%s",
            extensions=None,
        ),
        premat=dict(
            argstr="--premat=%s",
            extensions=None,
        ),
        reference=dict(
            argstr="--ref=%s",
            extensions=None,
            mandatory=True,
            position=1,
        ),
        relwarp=dict(
            argstr="--rel",
            xor=["abswarp"],
        ),
        shift_direction=dict(
            argstr="--shiftdir=%s",
            requires=["shift_in_file"],
        ),
        shift_in_file=dict(
            argstr="--shiftmap=%s",
            extensions=None,
        ),
        warp1=dict(
            argstr="--warp1=%s",
            extensions=None,
        ),
        warp2=dict(
            argstr="--warp2=%s",
            extensions=None,
        ),
    )
    inputs = ConvertWarp.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_ConvertWarp_outputs():
    output_map = dict(
        out_file=dict(
            extensions=None,
        ),
    )
    outputs = ConvertWarp.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..matlab import MatlabCommand


def test_MatlabCommand_inputs():
    input_map = dict(
        args=dict(
            argstr="%s",
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        logfile=dict(
            argstr="-logfile %s",
            extensions=None,
        ),
        mfile=dict(
            usedefault=True,
        ),
        nodesktop=dict(
            argstr="-nodesktop",
            nohash=True,
            usedefault=True,
        ),
        nosplash=dict(
            argstr="-nosplash",
            nohash=True,
            usedefault=True,
        ),
        paths=dict(),
        postscript=dict(
            usedefault=True,
        ),
        prescript=dict(
            usedefault=True,
        ),
        script=dict(
            argstr='-r "%s;exit"',
            mandatory=True,
            position=-1,
        ),
        script_file=dict(
            extensions=None,
            usedefault=True,
        ),
        single_comp_thread=dict(
            argstr="-singleCompThread",
            nohash=True,
        ),
        uses_mcr=dict(
            nohash=True,
            xor=["nodesktop", "nosplash", "single_comp_thread"],
        ),
    )
    inputs = MatlabCommand.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..anisotropic_power import APMQball


def test_APMQball_inputs():
    input_map = dict(
        b0_thres=dict(
            usedefault=True,
        ),
        in_bval=dict(
            extensions=None,
            mandatory=True,
        ),
        in_bvec=dict(
            extensions=None,
            mandatory=True,
        ),
        in_file=dict(
            extensions=None,
            mandatory=True,
        ),
        mask_file=dict(
            extensions=None,
        ),
        out_prefix=dict(),
    )
    inputs = APMQball.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_APMQball_outputs():
    output_map = dict(
        out_file=dict(
            extensions=None,
        ),
    )
    outputs = APMQball.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..utils import ComputeTDI


def test_ComputeTDI_inputs():
    input_map = dict(
        args=dict(
            argstr="%s",
        ),
        contrast=dict(
            argstr="-constrast %s",
        ),
        data_type=dict(
            argstr="-datatype %s",
        ),
        dixel=dict(
            argstr="-dixel %s",
            extensions=None,
        ),
        ends_only=dict(
            argstr="-ends_only",
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        fwhm_tck=dict(
            argstr="-fwhm_tck %f",
        ),
        in_file=dict(
            argstr="%s",
            extensions=None,
            mandatory=True,
            position=-2,
        ),
        in_map=dict(
            argstr="-image %s",
            extensions=None,
        ),
        map_zero=dict(
            argstr="-map_zero",
        ),
        max_tod=dict(
            argstr="-tod %d",
        ),
        nthreads=dict(
            argstr="-nthreads %d",
            nohash=True,
        ),
        out_file=dict(
            argstr="%s",
            extensions=None,
            position=-1,
            usedefault=True,
        ),
        precise=dict(
            argstr="-precise",
        ),
        reference=dict(
            argstr="-template %s",
            extensions=None,
        ),
        stat_tck=dict(
            argstr="-stat_tck %s",
        ),
        stat_vox=dict(
            argstr="-stat_vox %s",
        ),
        tck_weights=dict(
            argstr="-tck_weights_in %s",
            extensions=None,
        ),
        upsample=dict(
            argstr="-upsample %d",
        ),
        use_dec=dict(
            argstr="-dec",
        ),
        vox_size=dict(
            argstr="-vox %s",
            sep=",",
        ),
    )
    inputs = ComputeTDI.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_ComputeTDI_outputs():
    output_map = dict(
        out_file=dict(
            extensions=None,
        ),
    )
    outputs = ComputeTDI.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-

import os
from ...testing import example_data

minc2Dfile = example_data("minc_test_2D_00.mnc")
minc3Dfile = example_data("minc_test_3D_00.mnc")

nlp_config = example_data("minc_nlp.conf")


def nonempty_minc_data(i, shape="2D"):
    return example_data("minc_test_%s_%.2d.mnc" % (shape, i))



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""
Package contains interfaces for using existing functionality in other packages

Requires Packages to be installed
"""

from .base import IdentityInterface, Rename, Select, Split, Merge, AssertEqual
from .csv import CSVReader
from .wrappers import Function



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..brainsuite import Skullfinder


def test_Skullfinder_inputs():
    input_map = dict(
        args=dict(
            argstr="%s",
        ),
        bgLabelValue=dict(
            argstr="--bglabel %d",
        ),
        brainLabelValue=dict(
            argstr="--brainlabel %d",
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        inputMRIFile=dict(
            argstr="-i %s",
            extensions=None,
            mandatory=True,
        ),
        inputMaskFile=dict(
            argstr="-m %s",
            extensions=None,
            mandatory=True,
        ),
        lowerThreshold=dict(
            argstr="-l %d",
        ),
        outputLabelFile=dict(
            argstr="-o %s",
            extensions=None,
            genfile=True,
        ),
        performFinalOpening=dict(
            argstr="--finalOpening",
        ),
        scalpLabelValue=dict(
            argstr="--scalplabel %d",
        ),
        skullLabelValue=dict(
            argstr="--skulllabel %d",
        ),
        spaceLabelValue=dict(
            argstr="--spacelabel %d",
        ),
        surfaceFilePrefix=dict(
            argstr="-s %s",
        ),
        upperThreshold=dict(
            argstr="-u %d",
        ),
        verbosity=dict(
            argstr="-v %d",
        ),
    )
    inputs = Skullfinder.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_Skullfinder_outputs():
    output_map = dict(
        outputLabelFile=dict(
            extensions=None,
        ),
    )
    outputs = Skullfinder.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..gtract import gtractTensor


def test_gtractTensor_inputs():
    input_map = dict(
        applyMeasurementFrame=dict(
            argstr="--applyMeasurementFrame ",
        ),
        args=dict(
            argstr="%s",
        ),
        b0Index=dict(
            argstr="--b0Index %d",
        ),
        backgroundSuppressingThreshold=dict(
            argstr="--backgroundSuppressingThreshold %d",
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        ignoreIndex=dict(
            argstr="--ignoreIndex %s",
            sep=",",
        ),
        inputVolume=dict(
            argstr="--inputVolume %s",
            extensions=None,
        ),
        maskProcessingMode=dict(
            argstr="--maskProcessingMode %s",
        ),
        maskVolume=dict(
            argstr="--maskVolume %s",
            extensions=None,
        ),
        medianFilterSize=dict(
            argstr="--medianFilterSize %s",
            sep=",",
        ),
        numberOfThreads=dict(
            argstr="--numberOfThreads %d",
        ),
        outputVolume=dict(
            argstr="--outputVolume %s",
            hash_files=False,
        ),
        resampleIsotropic=dict(
            argstr="--resampleIsotropic ",
        ),
        size=dict(
            argstr="--size %f",
        ),
    )
    inputs = gtractTensor.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_gtractTensor_outputs():
    output_map = dict(
        outputVolume=dict(
            extensions=None,
        ),
    )
    outputs = gtractTensor.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..utils import FixTopology


def test_FixTopology_inputs():
    input_map = dict(
        args=dict(
            argstr="%s",
        ),
        copy_inputs=dict(
            mandatory=True,
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        ga=dict(
            argstr="-ga",
        ),
        hemisphere=dict(
            argstr="%s",
            mandatory=True,
            position=-1,
        ),
        in_brain=dict(
            extensions=None,
            mandatory=True,
        ),
        in_inflated=dict(
            extensions=None,
            mandatory=True,
        ),
        in_orig=dict(
            extensions=None,
            mandatory=True,
        ),
        in_wm=dict(
            extensions=None,
            mandatory=True,
        ),
        mgz=dict(
            argstr="-mgz",
        ),
        seed=dict(
            argstr="-seed %d",
        ),
        sphere=dict(
            argstr="-sphere %s",
            extensions=None,
        ),
        subject_id=dict(
            argstr="%s",
            mandatory=True,
            position=-2,
            usedefault=True,
        ),
        subjects_dir=dict(),
    )
    inputs = FixTopology.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_FixTopology_outputs():
    output_map = dict(
        out_file=dict(
            extensions=None,
        ),
    )
    outputs = FixTopology.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..minc import Convert


def test_Convert_inputs():
    input_map = dict(
        args=dict(
            argstr="%s",
        ),
        chunk=dict(
            argstr="-chunk %d",
        ),
        clobber=dict(
            argstr="-clobber",
            usedefault=True,
        ),
        compression=dict(
            argstr="-compress %s",
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        input_file=dict(
            argstr="%s",
            extensions=None,
            mandatory=True,
            position=-2,
        ),
        output_file=dict(
            argstr="%s",
            extensions=None,
            genfile=True,
            hash_files=False,
            name_source=["input_file"],
            name_template="%s_convert_output.mnc",
            position=-1,
        ),
        template=dict(
            argstr="-template",
        ),
        two=dict(
            argstr="-2",
        ),
    )
    inputs = Convert.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_Convert_outputs():
    output_map = dict(
        output_file=dict(
            extensions=None,
        ),
    )
    outputs = Convert.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..io import FreeSurferSource


def test_FreeSurferSource_inputs():
    input_map = dict(
        hemi=dict(
            usedefault=True,
        ),
        subject_id=dict(
            mandatory=True,
        ),
        subjects_dir=dict(
            mandatory=True,
        ),
    )
    inputs = FreeSurferSource.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_FreeSurferSource_outputs():
    output_map = dict(
        BA_stats=dict(
            altkey="BA",
            loc="stats",
        ),
        T1=dict(
            extensions=None,
            loc="mri",
        ),
        annot=dict(
            altkey="*annot",
            loc="label",
        ),
        aparc_a2009s_stats=dict(
            altkey="aparc.a2009s",
            loc="stats",
        ),
        aparc_aseg=dict(
            altkey="aparc*aseg",
            loc="mri",
        ),
        aparc_stats=dict(
            altkey="aparc",
            loc="stats",
        ),
        area_pial=dict(
            altkey="area.pial",
            loc="surf",
        ),
        aseg=dict(
            extensions=None,
            loc="mri",
        ),
        aseg_stats=dict(
            altkey="aseg",
            loc="stats",
        ),
        avg_curv=dict(
            loc="surf",
        ),
        brain=dict(
            extensions=None,
            loc="mri",
        ),
        brainmask=dict(
            extensions=None,
            loc="mri",
        ),
        curv=dict(
            loc="surf",
        ),
        curv_pial=dict(
            altkey="curv.pial",
            loc="surf",
        ),
        curv_stats=dict(
            altkey="curv",
            loc="stats",
        ),
        entorhinal_exvivo_stats=dict(
            altkey="entorhinal_exvivo",
            loc="stats",
        ),
        filled=dict(
            extensions=None,
            loc="mri",
        ),
        graymid=dict(
            altkey=["graymid", "midthickness"],
            loc="surf",
        ),
        inflated=dict(
            loc="surf",
        ),
        jacobian_white=dict(
            loc="surf",
        ),
        label=dict(
            altkey="*label",
            loc="label",
        ),
        norm=dict(
            extensions=None,
            loc="mri",
        ),
        nu=dict(
            extensions=None,
            loc="mri",
        ),
        orig=dict(
            extensions=None,
            loc="mri",
        ),
        pial=dict(
            loc="surf",
        ),
        rawavg=dict(
            extensions=None,
            loc="mri",
        ),
        ribbon=dict(
            altkey="*ribbon",
            loc="mri",
        ),
        smoothwm=dict(
            loc="surf",
        ),
        sphere=dict(
            loc="surf",
        ),
        sphere_reg=dict(
            altkey="sphere.reg",
            loc="surf",
        ),
        sulc=dict(
            loc="surf",
        ),
        thickness=dict(
            loc="surf",
        ),
        volume=dict(
            loc="surf",
        ),
        white=dict(
            loc="surf",
        ),
        wm=dict(
            extensions=None,
            loc="mri",
        ),
        wmparc=dict(
            extensions=None,
            loc="mri",
        ),
        wmparc_stats=dict(
            altkey="wmparc",
            loc="stats",
        ),
    )
    outputs = FreeSurferSource.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..utils import CenterMass


def test_CenterMass_inputs():
    input_map = dict(
        all_rois=dict(
            argstr="-all_rois",
        ),
        args=dict(
            argstr="%s",
        ),
        automask=dict(
            argstr="-automask",
        ),
        cm_file=dict(
            argstr="> %s",
            extensions=None,
            hash_files=False,
            keep_extension=False,
            name_source="in_file",
            name_template="%s_cm.out",
            position=-1,
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        in_file=dict(
            argstr="%s",
            copyfile=True,
            extensions=None,
            mandatory=True,
            position=-2,
        ),
        local_ijk=dict(
            argstr="-local_ijk",
        ),
        mask_file=dict(
            argstr="-mask %s",
            extensions=None,
        ),
        roi_vals=dict(
            argstr="-roi_vals %s",
        ),
        set_cm=dict(
            argstr="-set %f %f %f",
        ),
    )
    inputs = CenterMass.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_CenterMass_outputs():
    output_map = dict(
        cm=dict(),
        cm_file=dict(
            extensions=None,
        ),
        out_file=dict(
            extensions=None,
        ),
    )
    outputs = CenterMass.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""The fsl module provides classes for interfacing with the `FSL
<http://www.fmrib.ox.ac.uk/fsl/index.html>`_ command line tools.  This
was written to work with FSL version 4.1.4.

These are the base tools for working with FSL.
Preprocessing tools are found in fsl/preprocess.py
Model tools are found in fsl/model.py
DTI tools are found in fsl/dti.py

XXX Make this doc current!

Currently these tools are supported:

* BET v2.1: brain extraction
* FAST v4.1: segmentation and bias correction
* FLIRT v5.5: linear registration
* MCFLIRT: motion correction
* FNIRT v1.0: non-linear warp

Examples
--------
See the docstrings of the individual classes for examples.

"""

from glob import glob
import os

from ... import logging
from ...utils.filemanip import fname_presuffix
from ..base import traits, isdefined, CommandLine, CommandLineInputSpec, PackageInfo
from ...external.due import BibTeX

IFLOGGER = logging.getLogger("nipype.interface")


class Info(PackageInfo):
    """
    Handle FSL ``output_type`` and version information.

    output type refers to the type of file fsl defaults to writing
    eg, NIFTI, NIFTI_GZ

    Examples
    --------

    >>> from nipype.interfaces.fsl import Info
    >>> Info.version()  # doctest: +SKIP
    >>> Info.output_type()  # doctest: +SKIP


    """

    ftypes = {
        "NIFTI": ".nii",
        "NIFTI_PAIR": ".img",
        "NIFTI_GZ": ".nii.gz",
        "NIFTI_PAIR_GZ": ".img.gz",
    }

    if os.getenv("FSLDIR"):
        version_file = os.path.join(os.getenv("FSLDIR"), "etc", "fslversion")

    @staticmethod
    def parse_version(raw_info):
        return raw_info.splitlines()[0]

    @classmethod
    def output_type_to_ext(cls, output_type):
        """Get the file extension for the given output type.

        Parameters
        ----------
        output_type : {'NIFTI', 'NIFTI_GZ', 'NIFTI_PAIR', 'NIFTI_PAIR_GZ'}
            String specifying the output type.

        Returns
        -------
        extension : str
            The file extension for the output type.
        """

        try:
            return cls.ftypes[output_type]
        except KeyError:
            msg = "Invalid FSLOUTPUTTYPE: ", output_type
            raise KeyError(msg)

    @classmethod
    def output_type(cls):
        """Get the global FSL output file type FSLOUTPUTTYPE.

        This returns the value of the environment variable
        FSLOUTPUTTYPE.  An exception is raised if it is not defined.

        Returns
        -------
        fsl_ftype : string
            Represents the current environment setting of FSLOUTPUTTYPE
        """
        try:
            return os.environ["FSLOUTPUTTYPE"]
        except KeyError:
            IFLOGGER.warning(
                "FSLOUTPUTTYPE environment variable is not set. "
                "Setting FSLOUTPUTTYPE=NIFTI"
            )
            return "NIFTI"

    @staticmethod
    def standard_image(img_name=None):
        """Grab an image from the standard location.

        Returns a list of standard images if called without arguments.

        Could be made more fancy to allow for more relocatability"""
        try:
            fsldir = os.environ["FSLDIR"]
        except KeyError:
            raise Exception("FSL environment variables not set")
        stdpath = os.path.join(fsldir, "data", "standard")
        if img_name is None:
            return [
                filename.replace(stdpath + "/", "")
                for filename in glob(os.path.join(stdpath, "*nii*"))
            ]
        return os.path.join(stdpath, img_name)


class FSLCommandInputSpec(CommandLineInputSpec):
    """
    Base Input Specification for all FSL Commands

    All command support specifying FSLOUTPUTTYPE dynamically
    via output_type.

    Example
    -------
    fsl.ExtractRoi(tmin=42, tsize=1, output_type='NIFTI')
    """

    output_type = traits.Enum("NIFTI", list(Info.ftypes.keys()), desc="FSL output type")


class FSLCommand(CommandLine):
    """Base support for FSL commands."""

    input_spec = FSLCommandInputSpec
    _output_type = None

    references_ = [
        {
            "entry": BibTeX(
                "@article{JenkinsonBeckmannBehrensWoolrichSmith2012,"
                "author={M. Jenkinson, C.F. Beckmann, T.E. Behrens, "
                "M.W. Woolrich, and S.M. Smith},"
                "title={FSL},"
                "journal={NeuroImage},"
                "volume={62},"
                "pages={782-790},"
                "year={2012},"
                "}"
            ),
            "tags": ["implementation"],
        }
    ]

    def __init__(self, **inputs):
        super(FSLCommand, self).__init__(**inputs)
        self.inputs.on_trait_change(self._output_update, "output_type")

        if self._output_type is None:
            self._output_type = Info.output_type()

        if not isdefined(self.inputs.output_type):
            self.inputs.output_type = self._output_type
        else:
            self._output_update()

    def _output_update(self):
        self._output_type = self.inputs.output_type
        self.inputs.environ.update({"FSLOUTPUTTYPE": self.inputs.output_type})

    @classmethod
    def set_default_output_type(cls, output_type):
        """Set the default output type for FSL classes.

        This method is used to set the default output type for all fSL
        subclasses.  However, setting this will not update the output
        type for any existing instances.  For these, assign the
        <instance>.inputs.output_type.
        """

        if output_type in Info.ftypes:
            cls._output_type = output_type
        else:
            raise AttributeError("Invalid FSL output_type: %s" % output_type)

    @property
    def version(self):
        return Info.version()

    def _gen_fname(self, basename, cwd=None, suffix=None, change_ext=True, ext=None):
        """Generate a filename based on the given parameters.

        The filename will take the form: cwd/basename<suffix><ext>.
        If change_ext is True, it will use the extentions specified in
        <instance>intputs.output_type.

        Parameters
        ----------
        basename : str
            Filename to base the new filename on.
        cwd : str
            Path to prefix to the new filename. (default is os.getcwd())
        suffix : str
            Suffix to add to the `basename`.  (defaults is '' )
        change_ext : bool
            Flag to change the filename extension to the FSL output type.
            (default True)

        Returns
        -------
        fname : str
            New filename based on given parameters.

        """

        if basename == "":
            msg = "Unable to generate filename for command %s. " % self.cmd
            msg += "basename is not set!"
            raise ValueError(msg)
        if cwd is None:
            cwd = os.getcwd()
        if ext is None:
            ext = Info.output_type_to_ext(self.inputs.output_type)
        if change_ext:
            if suffix:
                suffix = "".join((suffix, ext))
            else:
                suffix = ext
        if suffix is None:
            suffix = ""
        fname = fname_presuffix(basename, suffix=suffix, use_ext=False, newpath=cwd)
        return fname

    def _overload_extension(self, value, name=None):
        return value + Info.output_type_to_ext(self.inputs.output_type)


def check_fsl():
    ver = Info.version()
    if ver:
        return 0
    else:
        return 1


def no_fsl():
    """Checks if FSL is NOT installed
    used with skipif to skip tests that will
    fail if FSL is not installed"""

    if Info.version() is None:
        return True
    else:
        return False


def no_fsl_course_data():
    """check if fsl_course data is present"""
    return not (
        "FSL_COURSE_DATA" in os.environ
        and os.path.isdir(os.path.abspath(os.environ["FSL_COURSE_DATA"]))
    )



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..model import SMM


def test_SMM_inputs():
    input_map = dict(
        args=dict(
            argstr="%s",
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        mask=dict(
            argstr='--mask="%s"',
            copyfile=False,
            extensions=None,
            mandatory=True,
            position=1,
        ),
        no_deactivation_class=dict(
            argstr="--zfstatmode",
            position=2,
        ),
        output_type=dict(),
        spatial_data_file=dict(
            argstr='--sdf="%s"',
            copyfile=False,
            extensions=None,
            mandatory=True,
            position=0,
        ),
    )
    inputs = SMM.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_SMM_outputs():
    output_map = dict(
        activation_p_map=dict(
            extensions=None,
        ),
        deactivation_p_map=dict(
            extensions=None,
        ),
        null_p_map=dict(
            extensions=None,
        ),
    )
    outputs = SMM.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value
